



#include "MiscCommandline.h"
#include "base58.h"
#include "commandLine.h"
#include "miniPriv.h"
#include "extraInfo.h"
#include "privKey.h"
#include "mnemonic.h"
#include "GenerationParameters.h"
#include "GenerationMethods.h"
#include "language.h"
#include "printPS.h"
#include "bitgenVersion.h"
#include "FileUtil.h"
#include "bitgenConfig.h"
#include "keyDerivationFunction.h"
#include "binaryTransaction.h"
#include "TransactionCommandline.h"


#include <iostream>
#include <list>
#include <stdlib.h>
#include <stdio.h>



int parseNumTextWords(const std::string& numWordsStr, std::string& wallet)
{
	if(numWordsStr == "jaxx")
	{
		wallet = "Jaxx";
		return 12;
	}
	else if(numWordsStr == "coinomi")
	{
		wallet = "Coinomi";		
		return 18;
	}
	else if(numWordsStr == "exodus")
	{
		wallet = "Exodus";		
		return 12;
	}
	else if(numWordsStr == "mycelium")
	{
		wallet = "Mycelium";
		return 12;
	}
	else if(numWordsStr == "multibit")
	{
		wallet = "Multibit HD";
		return 12;
	}
	
	
	const int numWords = atoi(numWordsStr.c_str());
	return numWords;
}






void listMnemonicSize(const std::vector<std::string>& args, const std::string& thetime)
{
	if(args.size() < 1)
	{
		std::cout << "Usage: bitgen listMnemonic" << std::endl;
		return;
	}	

	std::cout << "WALLET      Default Minsize  MaxSize      (TextWords)" << std::endl;
	std::cout << "=====================================================" << std::endl;	
	std::cout << "Coinomi:    18      12       24" << std::endl;
	std::cout << "Mycelium:   12                 " << std::endl;
	std::cout << "Exodus      12                 " << std::endl;
	std::cout << "Jaxx        12                 " << std::endl;
	std::cout << "MultibitHD  12      12       24" << std::endl;
	
	
	std::cout << std::endl;
}






void getMnemonicInformation(const std::vector<std::string>& args, const std::string& thetime)
{
	if(args.size() < 2)
	{
		std::cout << "Usage: bitgen mnemonicinfo MNEMONIC ..." << std::endl;
		return;
	}	

	std::string mnemonicStr(args[1]);
	std::cout << "Mnemonic: " << mnemonicStr << std::endl;
	
	const Mnemonic mnemonic(mnemonicStr/*, false*/); 
	
	const std::string privKey = mnemonic.getAsNumber();
	
	std::cout << "Private data (without checksum):     " << hexString(privKey) << std::endl;
	
	const ExtendedPrivateKey extPk = calculateMasterKey(privKey);
	
	/*
	std::cout << "##############################" << std::endl;
	const std::string seed(convertFromHex("000102030405060708090a0b0c0d0e0f"));
	calculateMasterKey(seed);
	
	std::cout << "!!!!!!!!!!!!!!!!!!!!!!!!" << std::endl;
	
	const std::string seed2(convertFromHex("fffcf9f6f3f0edeae7e4e1dedbd8d5d2cfccc9c6c3c0bdbab7b4b1aeaba8a5a29f9c999693908d8a8784817e7b7875726f6c696663605d5a5754514e4b484542"));
	calculateMasterKey(seed2);
	*/
}









void getCppString(const std::vector<std::string>& args)
{	
	if(args.size() < 2)
	{
		std::cout << "Usage: bitgen cppstr hextransaction" << std::endl;	
		return;
	}	
	
	std::cout << std::endl;
	
	const std::string hexStr(args[1]);
	
	
	//Determine if filename or hex string	
	//const std::string trans
	const BinaryTransaction trans = TransactionCommandline::getTrans(hexStr);
	
	const std::string hex = trans.asHex();
		
	std::cout << "Orig trans: " << std::endl;
	
	for(int i = 0 ; i < hex.size() ; i++)
	{
		if(i % 79 == 0)
		{
			std::cout << "\\\n";			
		}
				
		const unsigned char c = hex[i];		
		std::cout << c;
	}
	std::cout << "\n";				
}






void showAscii(const std::vector<std::string>& args)
{	
	if(args.size() < 2)
	{
		std::cout << "Usage: bitgen showascii hextransaction" << std::endl;	
		return;
	}	

	const std::string hexStr(args[1]);

	const std::string trans = convertFromHex(hexStr);

	for(int i = 0 ; i < trans.size() ; i++)
	{
		const unsigned char c = trans[i];
		
		if(c >= 32 &&  c <= 128)
		{
			std::cout << c;
		}
		else
		{
			std::cout << "_";
		}
	}
	
	std::cout << std::endl;
}








#ifndef NO_BRAINWALLET_GENERATION

void testArgon(const std::string& passphrase, const std::string& salt,
	const std::string& hashNumStr)
{

	std::cout << "Testing argon... " << std::flush;
	const std::string kdfStr = bitgenArgon2Kdf(passphrase, salt);
	const BigInt<512> hashNumComputed(kdfStr, 'c');
	
	const BigInt<512> hashNum(BigInt<512>::fromHex(hashNumStr)); //     hashNumStr.c_str());
	if(hashNum != hashNumComputed)
	{
		std::cout << std::endl;
		std::cout << "Error, computed kdfStr: " << hexString(kdfStr) << std::endl;
		
		throw std::string("Argon error");
	}

	std::cout << "OK" << std::endl;
	
}


void brainwalletTests()
{	
	const std::string passphrase("1234567890abcdefghijklmnopq");
	const std::string salt("someone@email.com");
	const std::string hashNumStr("9A5D21E6D7F1D3D1C5A7FBE2547F16ABF2DAE34379FA6309F9672491E2D83276");
	testArgon(passphrase, salt, hashNumStr);
	
	
	const std::string passphrase2("abc");
	const std::string salt2("this@abc.org");
	const std::string hashNumStr2("BD8BBBA3F95A59CA5E3100B4EFC488A9395A253F1366DFBCD84632A2F204C23D");	
	testArgon(passphrase2, salt2, hashNumStr2);
	
	
	const std::string passphrase3("");
	const std::string salt3("");
	const std::string hashNumStr3("93687FFD2E4E6A6A4E09188D6E1CCD3D981981597FE95237231A17DB6991E4F7");
	testArgon(passphrase3, salt3, hashNumStr3);
	
}
#endif





Language getLanguage()
{
	const std::string constPath = getConfigPath();
	
	const std::string confLang = getConfiguredLang();
	
	//std::cout << "Using lang: " << confLang << std::endl;
	
	if(confLang == "en")
	{
		return LANG_EN;		
	}
	else if(confLang == "de")
	{
		return LANG_DE;
	}
	
	
	return LANG_EN;
}




void setLanguage(int argc, char* argv[], const std::string& thetime)
{
	if(argc < 3)
	{
		std::cout << "Usage: bitgen setlang LANGUAGE" << std::endl;
		std::cout << "LANGUAGE: en de" << std::endl;
		return;			
	}

	const std::string lang(argv[2]);

	if(lang == "en")
	{ 
		std::cout << "Setting language to english" << std::endl;
	} 
	else if(lang == "de")
	{ 
		std::cout << "Setting language to deutch" << std::endl;
	}
	else
	{
		std::cout << "Unknown lang: " << lang << std::endl;
		return;
	}
		 
	configureLang(lang);
}




void printIntSize()
{
	std::cout << "sizeof(int)        : " << sizeof(int) << " gives " << sizeof(int)*8 << " bits" << std::endl;
	std::cout << "sizeof(long)       : " << sizeof(long) << " gives " << sizeof(long)*8 << " bits" << std::endl;
	std::cout << "sizeof(long long)  : " << sizeof(long long) << " gives " << sizeof(long long)*8 << " bits" << std::endl;
#if __x86_64__ || __ppc64__	
	std::cout << "sizeof(__uint128_t): " << sizeof(__uint128_t) << " gives " << sizeof(__uint128_t)*8 << " bits" << std::endl;	
#endif
}








void uncompressPublicKey(int argc, char* argv[], const std::string& thetime)
{
	if(argc < 3)
	{
		std::cout << "Usage: bitgen uncompress COMPRESSEDPUBKEY" << std::endl;
		return;			
	}

	const std::string pubkeyStr(argv[2]);
	//std::cout << "Will convert: " << pubkeyStr << std::endl;
	
	if(pubkeyStr.size() != 66)
	{
		std::cout << "Got key with size: " << pubkeyStr.size() << ", but need 66 chars" << std::endl;
		throw std::string("Error, compressed key size must be 66 charcters");
	}
	

	const PubKey pubKey = PubKey::fromHex(pubkeyStr);
	//const ECPoint pubKey = PubKey::fromHexString(pubkeyStr);
	
	const std::string uncompStr = hexString(pubKey.point.getRaw(false));
	
	std::cout << "Uncompressed: " << uncompStr << std::endl;
}




void compressPublicKey(int argc, char* argv[], const std::string& thetime)
{
	if(argc < 3)
	{
		std::cout << "Usage: bitgen compress VERBOSEPUBKEY" << std::endl;
		return;			
	}

	const std::string pubkeyStr(argv[2]);
	std::cout << "Will convert: " << pubkeyStr << std::endl;
	
	if(pubkeyStr.size() != 130)
	{
		std::cout << "Got key with size: " << pubkeyStr.size() << ", but need 130 chars" << std::endl;
		throw std::string("Error, verbose key size must be 130 charcters");
	}
	

	const PubKey pubKey = PubKey::fromHex(pubkeyStr);

	//const ECPoint pubKey = PubKey::fromHexString(pubkeyStr);
	
	/*
	if(!BitcoinEllipticCurve::verifyECPoint(pubKey))
	{
		throw std::string("Error, point not on curve");
	}
	
	std::cout << "OK, point on curve" << std::endl;
*/
	
	const std::string compStr = hexString(pubKey.point.getRaw(true));
	
	std::cout << "Compressed: " << compStr << std::endl;
}





void generatePaymentRequest(int argc, char* argv[], const std::string& thetime)
{
	if(argc < 3)
	{
		std::cout << "Usage: bitgen reqpayment coinAddress amount [label] [message] [LastDate]" << std::endl;
		return;			
	}
	
	const std::string adr(argv[2]);
	const PubAddress myPub(adr);
	//myPub.validate();

	std::string amount;
	if(argc > 3)
	{
		const std::string amountGiven(argv[3]); 
		//Make sure the value is a non-zero number
		const double value = atof(amountGiven.c_str());
		if(value == 0.0)
		{
			throw std::string("Incorrect or zero sum given");
		}
		
		std::cout << "Using amount: " << value << std::endl;
		char str[128];
		sprintf(str, "%0f", value);
		amount = str;
	}


	std::string label;
	if(argc > 4)
	{
		label = argv[4];
	}

	std::string message;
	if(argc > 5)
	{
		message = argv[5];
	}

	std::string lastDate;
	if(argc > 6)
	{
		lastDate = argv[6];
	}
		
	const std::list<std::string> not_used_printExtra;

	const Language lang = getLanguage();
	//const int margin(100);
	const PostscriptGenerator postscriptGenerator(versionNumber, applicationName, lang/*, margin*/);		
	const PublicPostscriptInfo pubInfo(myPub.toString() /*adr*/, lastDate, label);
	
	//postscriptGenerator.saveInvoiceFile(pubInfo, amount, message);
	const std::string psPublic = postscriptGenerator.generateInvoice(pubInfo, amount, message, thetime);
	
	const std::string pubFilename = pubInfo.pubkey + "_invoice.ps";
	std::cout << "Saving publfile: " << pubFilename << std::endl;
	fileWrite(pubFilename, psPublic);
	
}





void generatePublicPart(const std::vector<std::string>& args, int argc, char* argv[], const std::string& thetime)
{
	if(argc < 3)
	{
		std::cout << "Usage: bitgen pubaddr bitcoinAddress [label]" << std::endl;
		return;			
	}
	
	const std::string adr(argv[2]);
	const PubAddress myPub(adr);

	const std::vector<std::string> extraArgs = getExtraArgs(args, 1);
		
	std::string label;
	if(extraArgs.size() >= 1)
	{	
		label = extraArgs[0];
		std::cout << "Label: " << label << std::endl;
	}


	const Language lang = getLanguage();
	const PostscriptGenerator postscriptGenerator(versionNumber, applicationName, lang);
	const PublicPostscriptInfo pubInfo(myPub.toString() /*adr*/, thetime, label);
				
	const std::string psPublic = postscriptGenerator.postscriptPagePublic(pubInfo);
	
	const std::string pubFilename = myPub.toString() + "_share.ps";
	std::cout << "Saving publfile: " << pubFilename << std::endl;
	fileWrite(pubFilename, psPublic);	
}








void generateWifFromMiniPrivKey(int argc, char* argv[], const std::string& thetime)
{		
	const bool compressed(false);	
	
	if(argc < 3)
	{
		std::cout << "Usage: bitgen miniPrivKey MiniPrivateKey" << std::endl;
		throw std::string("Missing argument");
	}
		
	std::cout << "Generating bitcoin address from mini private key" << std::endl;

	
	//const 
	std::string miniAddr(argv[2]);
	if(miniAddr == "-")
	{
		std::cout << "Enter the wif value at one row" << std::endl;
		std::cout << "Press enter when all chars are entered" << std::endl;
		
		miniAddr = readStdinLine(30, false, isBase58Char);
	}
	else
	{
		std::cout << "WARNING: The information entered at the command line" << std::endl;
		std::cout << "WARNING: might be stored in a shell history file." << std::endl;
		std::cout << "WARNING: Consider using the \"-\" option for entering information." << std::endl;			
		std::cout << std::endl;		
	}
	
	
	std::cout << "Will convert mini address: " << miniAddr << std::endl;

	if(!isValidMiniPriv(miniAddr))
	{
		throw std::string("Incorrect mini addr");
	}

	//We now know the format is OK. Perform the conversion to standard key
	const BitcoinKeyPair keypair = convertMiniPriv(miniAddr, compressed);
		
		
	std::list<ExtraInfo> generationExtra;
	const ExtraInfo extraInfo("Mini private key", miniAddr);
	generationExtra.push_back(extraInfo);	
	//generationExtra.push_back("Mini private key: " + miniAddr);

	const PrivKey pk = keypair.privaddr.getPriv();
	//const std::string mnemonic = generateMnemonic(pk.key);
	const Mnemonic myMnemonicObj(pk.key); // = Mnemonic::generate(pk.key);
	//const std::string mnemonic = myMnemonicObj.toString();


	std::string accountName;
	
	GenerationParameters params(compressed, true, true);
	GenerationMethods::saveAllFiles(keypair.pubaddr.toString(), keypair.pubaddr.toFullCashAddrString(), keypair.pubaddr.toCashAddrReadableString(),
		keypair.privaddr.toString(), "Mini private key", 
		generationExtra, thetime, params, myMnemonicObj.toStr());
}
		




/*
void generatePostscriptKeypair(int argc, char* argv[], const std::string& thetime)
{
	if(argc < 3)
	{
		std::cout << "Usage: bitgen keypair wif [name]" << std::endl;
		return;			
	}

	const std::string wifStr(argv[2]);
	const Wif wif(wifStr);
	const PrivKey privKey = wif.getPriv();
	
	BitcoinEllipticCurve ec;
	
	const PubKey pubKey = privKey.getPubkey(ec);

	std::cout << "WIF : " << wif << std::endl;
	std::cout << "PKIF: " << pubKey.toString() << std::endl;

	
	const std::list<std::string> generationExtra;
	const Language lang = getLanguage();
	const PostscriptGenerator postscriptGenerator(versionNumber, applicationName, lang);
	const PublicPostscriptInfo pubInfo(pubKey.toString(), thetime, "");
	
	const PrivatePostscriptInfo privInfo(wif.toString(), "mnemonic", "GenMethod", privKey.compressed, generationExtra);
	
	postscriptGenerator.saveKeypairFile(privInfo,pubInfo);
	
}
*/




		/*
		const ChildNum num(0, false);
		const ExtendedPublicKey derived = CKDpub(pubKey, num);
		*/


/*
	//const std::string xpub("xpub661MyMwAqRbcFtXgS5sYJABqqG9YLmC4Q1Rdap9gSE8NqtwybGhePY2gZ29ESFjqJoCu1Rupje8YtGqsefD265TMg7usUDFdp6W1EGMcet8");

		//I = HMAC-SHA512(Key = cpar, Data = serP(point(kpar)) || ser32(i)).
		//const std::string data = 
		const BitcoinEllipticCurve e;
		const std::string serPoint = getExtendedPrivateKey.privKey.point(e).getRaw(true);
		
		BigInt<32> i(345);

	}
	
	std::cout << std::endl << "-----------------------------" << std::endl;
	const ExtendedKey extendedpubKey = decodeHD(xpub);
	std::cout << "Will get public HD" << std::endl;
	const ExtendedPublicKey getExtendedPublicKey = decodePublicHD(extendedpubKey);
	std::cout << "Extended public key: " << getExtendedPublicKey.toString() << std::endl;
	std::cout << "Extended public key ext format: " << getExtendedPublicKey.toExtFormat() << std::endl;
}
*/





/*
int getNumWords(const std::vector<std::string>& args, const int pos = 3)
{
	int numWords(24);
	if(args.size() > pos)
	{
		std::string numStr(args[pos]);
		//std::cout << "NUMWORDS: " << numStr << std::endl;
		numWords = atoi(numStr.c_str());
		if(numWords == 0)
		{
			std::cout << "Incorrect number: \"" << numStr << "\"" << std::endl;			
			throw std::string("Incorrect number argument");
		}
		else if(numWords > 24)
		{
			std::cout << "Max 24 seed words allowed: \"" << numStr << "\"" << std::endl;
			throw std::string("Incorrect number argument, too large");
		}
	}
		
	if(numWords < 12)
	{
		std::cout << "Mnemonic too short, mnemomic with " << numWords << " words is NOT SECURE" << std::endl;
		std::cout << std::endl;
		throw std::string("Mnemonic too short");
	}

	return numWords;
}
*/





/*
std::string getLastBytes(const BigInt<256>& privKey, const int numBytes)
{	
	const std::string privBytes = privKey.getBinaryLong();
	const int startPos = privBytes.size() - numBytes;
	const std::string priv = privBytes.substr(startPos);
	
	return priv;
}
*/



