
/**
 * hash.cpp - Bitcoin address generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
#include "hash.h"
#include "sha256.h"
#include "sha512.h"

#include <iostream>
#include <string.h>


Sha256::Sha256()
{
	sha256obj.init();		
}
	
void Sha256::add(const std::string& str)
{
	sha256obj.update((unsigned char*)str.c_str(), str.length());
}
	
std::string Sha256::getHash()
{
	unsigned char digest[SHA256::DIGEST_SIZE];
	sha256obj.final(digest);

	const std::string result((const char*)digest, SHA256::DIGEST_SIZE);
	return result;
}



std::string sha256(const std::string& input)
{
    unsigned char digest[SHA256::DIGEST_SIZE];
    memset(digest,0,SHA256::DIGEST_SIZE);
 
    SHA256 ctx = SHA256();
    ctx.init();
    ctx.update( (unsigned char*)input.c_str(), input.length());
    ctx.final(digest);
     
    const std::string result((const char*)digest, SHA256::DIGEST_SIZE);
    return result;
}


std::string doubleSha256(const std::string& input)
{
	return sha256(sha256(input));
}



std::string sha512(const std::string& input)
{
    unsigned char digest[SHA512::DIGEST_SIZE];
    memset(digest,0,SHA512::DIGEST_SIZE);
    SHA512 ctx = SHA512();
    ctx.init();
    ctx.update((unsigned char*)input.c_str(), input.length());
    ctx.final(digest);
     
    const std::string result((const char*)digest, SHA512::DIGEST_SIZE);
    return result;    
}



std::string ripemd160(const std::string& str)
{
	char hash[160];
	ripemd160(str.c_str(), str.size(), hash);

	std::string result(hash, 20);
	return result;
}

