#ifndef XMR_EC
#define XMR_EC

/**
 * bitcoinEC.h - Bitcoin generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
#include "bigint.h"
#include <iostream>

#include "xmrECPoint.h"
#include "util.h"




class XmrEllipticCurve
{
public:

	XmrEllipticCurve() : 
		n(Nval()),
		p(Pval()),		
		d("-98412DFC9311D490018C7338BF8688861767FF8FF5B2BEBE27548A14B235EC8FEDA4"),
		Gx("216936d3CD6E53fEC0A4E231FDD6DC5C692CC7609525A7B2C9562D608F25D51A"),
		Gy("6666666666666666666666666666666666666666666666666666666666666658"),
		genPoint(Gx, Gy),
		identityPoint(0, 1),
		one(1),
		two(2),
		three(3)
	{	
	}

	
	
	XmrECPoint addPoints(const XmrECPoint& p,  const XmrECPoint& pGen) const;		
	
	XmrECPoint multiplyGenPoint(const BigInt<2048>& ScalarHex) const;
	XmrECPoint multiplyPoint(const BigInt<2048>& val, const XmrECPoint& startPoint) const;

	XmrECPoint recoverPoint(const BigInt<257>& pubVal);
	
	//Todo: Method bool isPointOnCurve()
	
	static const char* Nval() { return "1000000000000000000000000000000014DEF9DEA2F79CD65812631A5CF5D3ED"; }
	
	//2^255 - 19
	static const char* Pval() { return "7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFED"; } 
	
	const BigInt<2048> n;
	const BigInt<2048> p;
	
	const BigInt<2048> d;
	
	const BigInt<2048> Gx;
	const BigInt<2048> Gy;

	const XmrECPoint genPoint;
	const XmrECPoint identityPoint;


	const BigInt<2048> zero;
	const BigInt<2048> one;
	const BigInt<2048> two;
	const BigInt<2048> three;

	static BigInt<256> getEdwards25519Max(const BigInt<256>& privKey);

private:	
	BigInt<2048> modinv(const BigInt<2048>& a, const BigInt<2048>& n) const;
	
	BigInt<2048> pow_mod(BigInt<2048> x, const BigInt<2048>& y) const;


	void calculateConstants() const;

	BigInt<2048> recoverX(const BigInt<2048>& y, const bool xLsb) const;
	BigInt<2048> toRange(const BigInt<2048>& val) const;
	BigInt<2048> makePositive(const BigInt<2048>& val) const;
};


template <int numBitsIn>
BigInt<257> reverseIntBytes(const BigInt<numBitsIn>& pubSpend)
{
	const BigInt<256> pubSpend256(pubSpend);
	const std::string pubSpend256Str = pubSpend256.getBinaryLong();
	const std::string hashViewKeyReversed = reverseStr(pubSpend256Str);
	const BigInt<256> hashAsInt(BigInt<256>::fromBytes(hashViewKeyReversed));
	const BigInt<257> myPubSpendVal(BigInt<257>::asPositive(hashAsInt));
	
	return myPubSpendVal;
}



#endif

