/**
 * bitcoin.cpp - Bitcoin address generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
/**
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE. 
*/
 



#include "pubXmrAddress.h"
#include "xmrMnemonic.h"
#include "test/unittestMonero.h"
#include "bigintutil.h"
#include "util.h"
#include "xmrPrivKey.h"


#include <iostream>


void unittestXmrMnemonic(const std::string& privHex, const std::string& mnemonicShould)
{
	const BigInt<256> privKeyExtended(privHex.c_str());		
				
	const XmrPrivKey privkey(privKeyExtended);
	const XmrMnemonic mnemonicObj(privkey);

	const std::string myMnemonic = mnemonicObj.toStr();

	std::cout << "Mnemonic: " << myMnemonic << std::endl;
	
	if(myMnemonic != mnemonicShould)
	{
		std::cout << std::endl;
		std::cout << "Error for: " << privHex << std::endl;
		std::cout << std::endl;
		std::cout << "Got   : " << myMnemonic << std::endl;
		std::cout << "Should: " << mnemonicShould << std::endl;
		std::cout << std::endl;
		
		throw std::string("Error in xmr test, mnemonic does not match");
	}
}



void testXmrMenmonicGeneration()
{
	unittestXmrMnemonic("237846237846278346823764278364782364", 
		"farming symptoms duckling roster dying jewels online gone logic yacht argue giddy nozzle nuisance nuisance abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey");

	unittestXmrMnemonic("e3741727bf289f166a49f928692c9bfe3a192b63a93c73af5c159ce6b9fdd303", 
		"educated ledge dolphin left sushi dating unnoticed sulking lava ruffled incur pairing mocked virtual gown cease origin alley ozone plywood initiate quote amaze vector origin");	

	unittestXmrMnemonic("209085bc2f135fa0733fc504c773fd64173b4e7ad950a6fddd337c70dd700001", 
		"fawns jump gadget etched vigilant southern jockey kennel foxes jogger total waxing badge school mice rebel also liar bulb tissue always ungainly mobile pliers jogger");	

	unittestXmrMnemonic("a4389f172f76e97f6e6f124df9272d300e3ba879008eac4b33b69c8d61795d05", 
		"hockey vane fonts spud were cousin jewels womanly worry soil bluntly cuisine pouch riots rage scamper mews went alley easy inmate aptitude claim soda claim");	

	unittestXmrMnemonic("fde771ebfee4552f0b66e53a3c353657333554eb97b002d42e2d34aa83d23b04", 
		"fabrics shuffled feel dogs dyslexic impel tufts jeers ajar darted business germs decay elite layout dogs nanny oilfield awful mouth moisture itinerary ouch opposite tufts");	

	unittestXmrMnemonic("6dc72ccb621e2b836b480cc9d0ec9d34db942f2c67e2b318fa908d43967a450f", 
		"ruthless ouch dove vague vats unzip lurk rugged catch lower tycoon sack popular jester enraged reunion lids uphill vague mowing vegan pram fight ringing dove");	

	unittestXmrMnemonic("2744a525b1ede8420b949e1fc4a4a20cd7fa5140e1c2f4d349070b516f00f607", 
		"taxi eclipse plus nautical wildly enraged doctor suffice rally oscar cigar zapped hippo drying wrist amidst memoir nightly wiring apricot saved fifteen later owed suffice");	

	unittestXmrMnemonic("165fee1f72e1647e6b90df6276d8580f9a59d8198862fd935e6a44d322613d0c", 
		"firm keep oasis robot tinted dusted different playful diet delayed pheasants enlist linen pencil biscuit tasked inline tapestry tamper system fading lagoon jump moment diet");	

	unittestXmrMnemonic("3ba03d76fc35c30079809d1f859ad074da08b44d59101955336e5967c3e31e0e", 
		"roared serving memoir exquisite pancakes taxi wife bypass left point else beer unsafe reunion edited soil stunning fences splendid enforce edgy tilt dummy jukebox unsafe");	

	std::cout << std::endl;
}




void testXmrPubkeyGeneration(const std::string& addrStr, const std::string& pubSpendHexShould)
{
	std::cout << "Testing pubkey generation: " << addrStr << std::flush;

	PubXmrAddress pubAddress(addrStr);

	const BigInt<257> pubSpend = pubAddress.getPubSpend();
	const std::string pubSpendHex = hexString(pubSpend.getBinary());

	if(pubSpendHex != pubSpendHexShould)
	{
		std::cout << std::endl;
		std::cout << "Got   : " << pubSpendHex << std::endl;
		std::cout << "Should: " << pubSpendHexShould << std::endl;
		std::cout << std::endl;
		
		throw std::string("XMR pubaddress error");
	}
	
	
	//const BigInt<257> pubView = pubXmrAddress.getPubView();
	
	std::cout << ".....OK" << std::endl;
}




void testXmrPubViewGeneration(const std::string& addrStr, const std::string& pubViewHexShould)
{

	PubXmrAddress pubAddress(addrStr);

	
	const BigInt<257> pubView = pubAddress.getPubView();
	const std::string pubViewHex = hexString(pubView.getBinary());
	
	std::cout << "pubView: " << pubViewHex << std::endl;
	
	if(pubViewHex != pubViewHexShould)
	{
		std::cout << std::endl;
		std::cout << "Got   : " << pubViewHex << std::endl;
		std::cout << "Should: " << pubViewHexShould << std::endl;
		std::cout << std::endl;
		
		throw std::string("XMR pubview error");
	}	
}



void testXmrPublicKeys()
{
	std::cout << "Pub spend tests" << std::endl;
	testXmrPubkeyGeneration("447ApEECweKMWhNCyUfZL7W38Xg76eAw7B2ocprNVGwYLkjJRa64nVYYEXjk22hppZ7ty1DuFbeLj9htoPr7doTD3YcHtzG",
		"7607C9FDFA89D6F83B1E0B020EE86599ADB4310E59701BA27AF434D17B398B41");

	testXmrPubkeyGeneration("4AjB9NnRQXE5Wd4fkFwW6C1HzEm56hXa96bMSK1BHzM1RXFsZyfKAX2ZsnDtDZXi3RBFnHCMiEZtL3GBabeiR3WNKxuhWmi",
		"92ACA9430C271F6F214A9421583C41C001716E148D07C4F71AC5F5CFF06641F0");

	testXmrPubkeyGeneration("49JCpsMVBBoW4ephusXptkHbyJU4cfeAxfKLzwzNi6h13baLsT3o2htgbSDg6pqmg8ZXMuXFWiRgwKEtm8EA6ELaQhmU6Lq",
		"0F1C65F65D4D0014E5B55B1B99FF33436325066664B491C1ADFAA659438B81CA");

	testXmrPubkeyGeneration("4Aok8JdbbXQXnuo6V7ThvBGHo3RpYMeec1UwU5CXhvHoFw4xidSZBg7GrpbiSCtDicjDAxoWuaNkvCjxANAq9nd98aMmc91",
		"59C2D0AE552144E102295C555F6F05695BDC2D82F62F3717B81BB8F9548655F2");

	std::cout << "Pub view tests" << std::endl;

	testXmrPubViewGeneration("46y9W2XTPRuh2mBUMxYX6mGbNcftmXBhdG2B6b13JfXWX4TgCw4GMq7249BDEQStp9XcjRH37jwUsfTyyLr1uzXdG8ATm3H", 
		"86741E2FA9E5EDF7E5180001D77B9C0AB752EA60D4AD684D063EAF59839405B7");

	testXmrPubViewGeneration("49rQNF7udnBGh2bFBA6Lywhaw13u4Qb1BGxqbg5dF5AzRb4QWb8BuPz2rKLN7uMspxWV7MGCoHhQg3xeQLuNq34q1dzNz4w", 
		"05B6042B688C6CB111FD5DA40559F546B0C5AA547BEFE40F0B856E233D3B54FF");
	
}




void testXmrMnemonic(const std::string& mnemonic, const std::string& hexStr)
{	
	const XmrMnemonic m1(mnemonic);
	const BigInt<256> p1 = m1.toPrivKeyInt();

	
	
	const BigInt<256> c1(hexStr.c_str());
	
	//const std::string asBytes = c2.getBinaryLong();
	//const std::string asBytesRev = reverseStr(asBytes);

	//const BigInt<256> c1(BigInt<256>::fromBytes(asBytesRev));

	
	if(p1 != c1)
	{
		std::cout << "Error for " << hexStr << std::endl;
		std::cout << "Computed: " << p1 << std::endl;
		std::cout << "Expected: " << c1 << std::endl;
		
		throw std::string("Unit test error");
	}	
	
	//std::cout << "Ok: " << p1 << " == " << c1 << std::endl;
	
}



void testPubSpendAndViewToAddress(const std::string& pubSpend, const std::string& pubView, const std::string& address)
{
	std::cout << "Testing: " << address << " ......." << std::flush;
	
	const BigInt<257> ps(pubSpend.c_str());	
	const BigInt<257> pw(pubView.c_str());
	
	PubXmrAddress p1(reverseIntBytes<257>(ps), reverseIntBytes<257>(pw), false);
	
	if(p1.toString() != address)
	{
		std::cout << std::endl;
		std::cout << "Computed : " << p1.toString() << std::endl;
		std::cout << "Exptected: " << address << std::endl;
		
		throw std::string("Error in pub xmr from pubspend and pubview");
	}


	std::cout << "ok" << std::endl;
}



void testMenmonicToAddress(const std::string& mnemonicStr, const std::string& address)
{
	std::cout << "Testing: " << address << std::flush;
	
	const XmrMnemonic mnemonicObj(mnemonicStr);
	const XmrPrivKey myPrivKeySpend = mnemonicObj.getSpendKey();
	const XmrPrivKey myViewKey = myPrivKeySpend.privateSpendToView();
		
	XmrEllipticCurve ec;
	
	const XmrPubKey pubKeySpend = myPrivKeySpend.getPubkey(ec);	
	const XmrPubKey pubKeyView = myViewKey.getPubkey(ec);
	
	PubXmrAddress pubXmrAddress(pubKeySpend, pubKeyView);
	
	if(pubXmrAddress.toString() != address)
	{
		std::cout << "Incorrect address" << std::endl;
		std::cout << "Expected  : " << address << std::endl;		
		std::cout << "Calculated: " << pubXmrAddress.toString() << std::endl;
		throw std::string("Incorrect xmr address");
	}
	
	std::cout << "......OK" << std::endl;

	
}



void xmrUnittests()
{	
	testXmrMnemonic("succeed cadets equip worry hence enigma roomy ankle recipe cavernous younger ivory stunning dagger peeled border tamper nylon sunken scuba amused listen movement faked roomy",
		"BB38BAB14A0E6ACAC26AE57678743C5F707E4F7BA809CFA7E191C0421F6925E0");


	testXmrMnemonic("paper boxes upstairs border dauntless pimple fetches pride each auburn muzzle atlas syllabus camp emails anecdote bays cobra pager bounced sorry cafe strained lodge sorry",
		"B049F6C1AE0D91050FC4256F1B4FF312849B4ACD8CF1CB717A23CD66CA1A745E");


	std::cout << "Testing pub spend/view to address" << std::endl;
	
	
	testPubSpendAndViewToAddress("69EF133187DABE35075647A459E46BE5C24BA9C2DB2CB09E4827364F4F5D4D72", "087a83bffce5291bf3567adc1674d44c167db96695aef080aa40b46fa62c54e5",
		"45dwkxEfXH79sSn4qVz3FGfRx662UkRGbTUXsKk5ik1zL5GKSVWYySx5gA1D2asVnsDj9ZEhhmRuhNXDJh7CjxPVSsJvteR");
		
	testPubSpendAndViewToAddress("44595f0dfca6050f6f8404ed73236256aee393a2f5c46a7de2c717fceec6824e", "c273ffebbaed3cb4a1d9d03fea0fc2b7869790c51642a3726536a3ede8664cf4", 
		"44DLRxqBbSc3akCHjiUqAHFVwF8oRUy6qN4FXUtKzBBoEB4s59agKaKXDMpLP3vDjfXhRvQuhPuxnL8n9L4tCs4FUaCtA8C");

	testPubSpendAndViewToAddress("ef08218c7aae0a520df2e0c305928ab45bcb1f814bd735b89497b9eb2eda9b75", "7cd3d438b23d46531d8c1f76c5ed1062740c5f732a6590101f3d89137f546264", 
		"4AgV9PKkmrhEj2pLPC8CAmXAhqPAUrJZSXsfNEYVjB4NLen4hUfEi8dEuKmsK6Jf9qHU7zGRaLUGP3hQRmBYxPiyCPbny8W");

	testPubSpendAndViewToAddress("eccfb1c001450db7752e6104ac0eafa242a30d2167878b5a5bf15610a0c76d48", "e7795adfbe904239c9e555ab86c5961c84802a926858ad5e45f22ff5f137e80e", 
		"4Abc5d4d1Q8Xgmes5pyP7CU98SAWZVKbGG7bdACQbrHADCGCSBTGwjoAfd6afVPCs75mf4eE5A4HSGmZy4rxiRZH2cuZzYQ");

	testPubSpendAndViewToAddress("4e024a42753da7c4d2310dde993ae468a6732785ab71fcd548462195a07e1946", "37a1ffdbb7a03f6dfa7082a426bb7ad9c2e808e051fc19236369ff0794c7a457", 
		"44aZsZSLQ2iZvQq4nWtF6bJWEoZMQFF4wcg78V9tLPLCCkCN5HvinazKPvmg8XSMyKdRZMniA6Fkk6vK3qqDyComAt6M9AL");

	testPubSpendAndViewToAddress("711a8a7dd962b7fc6b7a345e55fd33cba8f3596e1bc7c53994f10076c48a9694", "b152b4e343403f1e224d1c618c4360ae1765e788f4a134838b26a000d20aee84", 
		"45uhmSiBJogjDnjn6ZeBMUb4ktjaFHmGpAdchvWb8PNdRsWJJH2Bir263La9wyMRmhW7uUDi9UnTdP18xrJZf6DsFvMgyCF");

	testPubSpendAndViewToAddress("496291eb1641783f186a019cb209795ca348617294d8b98661e837818177f2e2", "875da8b5b8d01836cb2b6b5569a296c627c96a8962c48d6395f070d317b3aaf7", 
		"44QQQzdNyF5BZ73CT4x93WGVhmeei3qMrPUgL7YnS2fTetcYf1xvAhDAAZsvKSBGSqa9Mdo8uUTxQHf79i5Y17EqUt39C6C");

	testPubSpendAndViewToAddress("6e9321f56e78b4b3175e1d3d343926854a9a548eed7998f06dbb0254d8610426", "f29e3b9846e302b15ab76c63b151754d4f17d7ea29fe1f1493b846162e20addd", 
		"45p9NoGYXCwWxQmQkGj1mFPJ6SGBVDAdDhDTcqrZm71m7WqpcRe2VwXWfZT3uB36X6DvzduZpUCZk4Sd8gLVehsJRyphZyM");

	testPubSpendAndViewToAddress("0ef14b757213438939da57bbdaa80bf38c971559dfbaf7893125a1058f6556fc", "24c04f66d16f9c1a4a3d13d8785da5601bc4403a57b549b24f085e84ad015246", 
		"42Bx8s76SEePxGK2SWJ7NShjjTte6couGPwwBC3nk9fKjB7Ha7ZLKTy5Q3fLTFiA1SH5Nc3AuMPULWppScAYM4zD8uP4Vx3");

	testPubSpendAndViewToAddress("7ec91d28f3699458dd95c25d24eb32bba2c277274f54bfb9d42d460e7d3eec74", "f0bdcf8ce6316e6221e1a7aa1e306fe7acb56087d173089e7e05d2c70e338b2e", 
		"46Rmwpvfcj5Fs7DzmpjcEMYPJXHZFdXmGY5mnzTSZugwLZUATh73r9BHR1PbGK19wLfkXzBknSUtPTWaGePx9cwt6H5LuzK");

	testPubSpendAndViewToAddress("56e52068039427b93573048582106e785bbad221e8fca3ae76891018a6475811", "df7095c0685432505dff0837ca4ddfacc25007de1e92c603ebc1687b42de110a", 
		"44v6g5qssVxXykvKSUzzF7M8dLRtGg1UWWBWaNrDyAPy3zPYTXHk8k1ESfQoSD1D8zVtynUAJL4Bo1f3CH2RbNvk2B7c2pj");

	testPubSpendAndViewToAddress("a45cc67ab99c5054227a4fcb546576a44b8f455c589f0d7c319e4ef1000552e1", "02223504ce5158728e38dc98031068f29cd0a07deca9c7c751d17fc7c877e784", 
		"47rNFqBggAKF5DHR3SQekVUUsPfezWh8YMmqU744dnZFeds38M3ZcY7LALH8uhBkmMhaeTDkM6B7UaLeh1xGxKeNFwLspcU");

	testPubSpendAndViewToAddress("73dcc833164eacfcefac7888b1738180e9845b7c1ae722fa9fd278e3028e9724", "908a697870b79dd1ed0fbac35b0d078d51ac473a405c5555259039578afbd172", 
		"461mTKAyR6TjJoJEndWPMzNZcMeuBDKiHivNSh7XGg3p77iud3hnGHSc7YddHZYDVgQdyMg7HtQH2FF2jnLbjLiCDxkhjqq");

	testPubSpendAndViewToAddress("d0230e190605582762cd9e957e1782232712759ab14e9317a204e53bcc396d78", "b97a5c0ade3576ac636670c39722a9bb7dd0ab3ebc1cf51a037ffa91659e62fa", 
		"49Waf2ry7YB7b6Q2YHh1hK6t2RRM4QeAE4xGabxGsN2YMCBPVuWVPp9VqPAyix2oLkYMuKaaoWvdJ5MNBXvRash3VJjUbMJ");

	std::cout << "Ready testing pub spend/view to address" << std::endl;
}


void testMnemonicSeedsToAddresses()
{
	testMenmonicToAddress("civilian names girth utmost gave debut guru fruit river emulate unwind galaxy jester darted patio poker unveil luggage journal sequence gossip pouch karate listen fruit", 
		"476CYzhetQRWcciqKSMkeLRCGF6jMfFG5g8X59p1tKjnS17hPVzC3UPKCVAkJFJQ6fT7GKE4b2SDSGGUpHhTrJW3JjoXNdj");


	testMenmonicToAddress("loincloth ourselves imagine usage earth dewdrop ramped happens amused vane sifting adventure apart jaunt history thaw okay sleepless orbit ponies history geyser lush mullet jaunt", 
		"41y5MPCrvowdqwE25vL8cxT3empcW9AQWbgTSk1XHMwheKYG47oSQ9HXiKETwTgoiHLZAjD31Sx1WeWzjKLUSJej4ktG6JC");

	testMenmonicToAddress("enhanced nuance navy aside jaws boss afar august woes zinger haystack dynamite chlorine fidget eldest actress goodbye anecdote ember vegan swung apply bagpipe bicycle ember", 
		"4B7q2UVHQvX7BbpzSsKvX7X7UB5SfXuFBatxQqTkAQUEZtR2WBeCopRcYQjCevqvo5QUBpd31qpZmf7HbwwvzGxALBrF7Sm");

	testMenmonicToAddress("waveform absorb liquid fabrics enhanced nudged fugitive cuisine cupcake sword waking cycling megabyte awning whale darted mouth itinerary shuffled daytime shelter suture junk lawsuit cuisine", 
		"47UoD3jBfasajxXXSKHHTjCKDJGWbMSLsdfEfhexQoiP5tEWjU4PaKrDP2T2hwZxdY4Z8JqGT267BYLu7aMu7k6VUdHUzW9");

	testMenmonicToAddress("elite solved hounded adjust medicate donuts lectures owner nostril zeal nimbly opus quote ruffled bluntly inexact royal navy diplomat fountain jeopardy petals mechanic moisture lectures", 
		"4B6FUMAth2ehG1MHLR5yL7YCyUbA1Kd1oRZLt5px7LrwitYuwvuQoVFAUHZrVum5Th8iTjjKM4FiefM7TGBg9LajGN2MygP");

	testMenmonicToAddress("seasons wipeout sizes bids inactive cuisine chlorine oval fight cousin laboratory masterful rogue having hawk nodes gained knuckle fight pockets fatal jazz sewage smash sizes", 
		"45WQxpWPDDRMEXp8rJpZY3Gejsm92DRhNKrKR53iczE95ZewPY8eWoSZk4Lteo3PGiDaaLmtXYuLoSVuNHXw6SA38YrhxSt");

	testMenmonicToAddress("incur injury bemused awning rumble donuts towel were merger rest hospital unopened aloof duration dwarf items remedy unopened menu eternal lumber awakened oval punch rest", 
		"4AC6TrD9ENi8YuqcAQRNGPfCYc3xZw4EhAL3GN221QkdWB4FLSJh1bR6W5BbnXXybQAPWUaZLvVQz3UkG9UBPKSSDSB4Q6h");

	testMenmonicToAddress("rogue succeed jetting zebra february wetsuit fainted mighty fudge whale tobacco dozen elope pylons gambit tedious hold soothe arbitrary rhythm bumper mostly nearby omission mostly", 
		"49fFhq1iiow7qjfqgQuPC2W9Bh19TxaA4imqDtbUtJMQ9jc29fG5JdVPp3bJ2QSAWF4zsLX7oWpXfZGr3rtpBRWqTC8dD2e");

	testMenmonicToAddress("nudged nightly axle musical hurried nugget mural yoga smelting jabbed album corrode comb omega cement twang ahead mayor sushi ruffled apply lion isolated jellyfish mayor", 
		"41iyVgkyJeQ1kn429fxGJV9iXzrMmvvUv54twAk78DfFinCtfFwCQzQJvtD6QDvwbuBctg156cu3dhHTRKPPfw881PgtPs9");

	testMenmonicToAddress("exotic setup identity lynx kennel metro under skirting olive cousin balding vigilant dullness judge hawk coils moat folding eating womanly fruit ruffled industrial knife hawk", 
		"43fADoPUtmjDUUYiDERi7KVA2zT9Bqp5NM3HiNqx1oN7EiXCswSns2922b27LPzVezRZ5BPeayayaeFwxBEUoRHNJJJSHYR");

	testMenmonicToAddress("yields winter fitting update assorted roles emit family timber dehydrate army dunes simplest idled dagger recipe nerves decay orders swiftly gossip liar origin paper timber", 
		"41fmsKriSsVYC5bMSWaXaZj9uirzaYtGZWwuvcznPzu8HqXyXa2J4EaersgEkBjKt26AUfVZ74YrCMnpQtHKpk7PTi8hJBN");
	
	
//------
	testMenmonicToAddress("abducts abducts abducts abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"44yQXfkWZNmJ8QgRfFWTzmJ8QgRfFWTzmJ8QgRfFWTzmJ9HskYAEKgjVy4kd3K4MaxERLtGa7FFrVNLF3jYWyjHCFA3tNHB");

	testMenmonicToAddress("ability ability ability abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey ability",
		"49GJVK3YQ7G3Q68xyoie3ASK9sBTXNue95yhG7PE7RLs6qtCWZpJ3NiUoUyBoqgURTKWYQumAc6EQYM4t4XQ7ZXyGidccNv");

	testMenmonicToAddress("ablaze ablaze ablaze abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"49gdEYzWxnW1LZAWrHVzwPTktCf4H51xqU3Q4ALZxmP448guEaFae3ZVPzGDTFGb55B7SQqgi4qgiB6ZVPxM1ZCs3HaeTEQ");

	testMenmonicToAddress("abnormal abnormal abnormal abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"43QZQAbeBoVeTUdzXtctEmDnQUe8mDu5PD2LaM4anysdD31HXxN1JfkT3vWH521aaPYb1HgxjaN783sNhsoUP1HCLXRjfqd");

	testMenmonicToAddress("abort abort abort abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4AdjxfpurQh3jkGXBaUCK19GEkqd6h4AfeYK9bfu4JT9eNE4jryjQMF8Bv51unxN4tccRb3WbgQCQ9fH246UJkJ9T3zN2GJ");

	testMenmonicToAddress("abrasive abrasive abrasive abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4AtVPQmP4s3ZLzkFWLBDtcij983t8rVbU5xUEbFR36Y4PNMHHftbMXXCSePKm7WynfDm54EgXAeLdNJWihn93QpVBq83G65");

	testMenmonicToAddress("absorb absorb absorb abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48cNbRvGyrb43yGQYLWKxshBnnq8nRkFfPcUwfqpihwJWdjWnnjg62j9SeskKjqkFnUefTJr547W1cuhAhvLZ6NfEbTrom7");

	testMenmonicToAddress("abyss abyss abyss abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48UKyjbjKHQ67eEekYbWoRNfvAnc6DU1JKUjwPVjqwYN6fFEZdfjg4NfVzoYn2sLqQbPVFwiZKDTY2zZLMNASGLDRGiNyFe");

	testMenmonicToAddress("academy academy academy abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48wBb9Q2357aKNwuqVYtCw6MMk3r34QB7Ww59dYCbVDvNH1u4Db2R45GHiLCqjKNmdUTG28rBxofYGGST5XSfYb6VCU7Pma");

	testMenmonicToAddress("aces aces aces abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"43Jt6a5FJ6sYA1NkT7LgPf7KpWPnYr9vBaXmPwsvxi5kf7nz6GQ4puzKVZtLRVJEFYXriMGVhv9ew2u9YFqhrGwY9bQMXEN");

	testMenmonicToAddress("aching aching aching abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"42MLmymi9SW9Ai1fXeg2Gf49S2CrTyPYJG9ru2hQcTWoW32BMtczP3jGP8mhFkYfhJ5jiwe38pLFYcewXyva94B6CR5Dux9");

	testMenmonicToAddress("acidic acidic acidic abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4B6MXWUe8bB9bgz8AscFSoFWHbGnuJqaUffJDhJG5N6jZ9gN5hNs56agZjW8csswSBM1yxwqjFRSuhsPEiot9dZiFU4QQzH");

	testMenmonicToAddress("acoustic acoustic acoustic abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"46UiKcKyTstPc15REsijpBAKZbq71AMFbM9ZsnwFMCdgRRyFTpCTPPPGeU4bsFQLv3NJtjYK5cG1LibP3v8Hjp5GGG12s4k");

	testMenmonicToAddress("acquire acquire acquire abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"43njs6Mz32YNyMfJtciYYoV4MATYRGD4d4beQkUy5sPDbXVRtKq22LWWwRnMi94tgS2RyFTGx4x9GJtJE3EAj4bT2YjWTyK");

	testMenmonicToAddress("across across across abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4A63K4AJY7WRrukQv2dQtCVoVfYvcbXbPEi6HHPWjZYeRVopLbCH8Vo98DGaHQ3s5VEs6b7WSVCj8EcHhA2Hv5oJB4J46pv");

	testMenmonicToAddress("actress actress actress abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4AXxoBCFJ5Fd9yMSo9ZYGhWWz6E7LgSwHTfrCM1AMMjKKtW6WGyabvh8yWjbx9ovRYVidG3ZczgNJ5t9jvDJAoj7TvqUmqb");

	testMenmonicToAddress("acumen acumen acumen abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey acumen",
		"41oY5nowso9bHqERwPzwoe4y64iaJmAMohe9btosNZJkKcmpX93KJSkXcNMNbk8b2oS23wpmfu4PVHfvnfgk3iQcAkzLjbt");

	testMenmonicToAddress("adapt adapt adapt abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"44TGgXAm7zV9yrnRKdZqbEYZ7oJQ5q9G3N1T2N52NWLQ8outhVXT1hDBYxpHCEgbVTeCD38u5kYeMGFzrvJRJvJnCUhnUXk");

	testMenmonicToAddress("addicted addicted addicted abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48eTXriPsDXNwhSKvrPw9H8rX37uiqyj96jYW8c32D11BezLxr5Ef1g25qupmqcz3ben14CLc3i9cJrUEsGc6PG2RDgPfdC");

	testMenmonicToAddress("adept adept adept abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"43ky6ViqM7ve6177BrqEnUYwLmkHvdCvmGpsPoiMsKy5Rbn4LARvBLYeQ4ApcbDqZN7NFFD9bQTM7GHRuf7y2ffnL91A4cx");

	testMenmonicToAddress("adhesive adhesive adhesive abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"47cjJf4XbU1jj1izcnRJBoW94HxfSG5Ppho6ZXzFidHHJBVh8vXSTedbb17PuY6HZGLGqwcXbX2cJ8WcW3bTW4osBvukKkN");

	testMenmonicToAddress("adjust adjust adjust abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"43149oKRhjPVFX79Cv5uMfbQa8ss7GKDKMnUmZx5xW7Q29hHSjGP1gA7nFPkSYiC2MTyqqEhyHhWRJYbncN4oBdUPvJLWmZ");

	testMenmonicToAddress("adopt adopt adopt abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4AfsKr7vm6cJB2NATvAwSn1g2eddxrbPTDyrrNY2Pp51UH83tAYALA29uvvMxnM54872PGAdRDNonfG25Vq9TMXQSX1DV58");

	testMenmonicToAddress("adrenalin adrenalin adrenalin abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4AmrZ2KZ7GufeJ5unveSXQeSAqsoSYDo7KKf2hxwtY4DecJpR6y8PFoAnumErZHiqhEnaocPBxZnRCJighvjjoSYUKrQxXB");

	testMenmonicToAddress("adult adult adult abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"49YAngQDTB76x4HPoXnq9YRXMX9XjhfoL2FG1uHJsy68hKuZCtPaZu8JKDyEVCoaGaUtUA936kFwB67yRbeDJS8cGa2RLg7");

	testMenmonicToAddress("adventure adventure adventure abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"425iTC4QLVt3xbp6qAPgF9amfNgmT4f7Ng73JFcsq7qogYC7HfkTR9eJxRnpHy2pZBHNLY7osR6mFVw4qR5QcayCHXrfEcA");

	testMenmonicToAddress("aerial aerial aerial abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"43ncFGydWrtYAMqUgLdQWDAXc5qbdWgJwNd7ZCye7N5vf6B3pX6qZCFNTqHpssoLfdN3RrpvSosVZDPvrDniAPcH6Fpeakn");

	testMenmonicToAddress("afar afar afar abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48kBrbEaC5d5KSfKr3dhZaK9pBvNBgnN4Rjyk7DHgnYshfhzB1HhCCBfNCRa1ChCqe1xjXVnvALat6QRwpt6wbeCFXnHyvs");

	testMenmonicToAddress("affair affair affair abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey affair",
		"479VjKk8Y9XKFAPFMe47jRNZ7ErgPeKgAU4ZADMTCPuMbSe8zT3nL7dcUYwW9S9xjTda93nHCYv4Si4SjR7vWUs4LLF3fs4");

	testMenmonicToAddress("afield afield afield abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4AJLE4hjiMRUTF1ZXnTCEaW22Pw3HsYcLh4Fbe1yiSGN9rVjFPwvsFDfUT6BKTdUpYWHAaJFfxCdUebwQNXAYSeuS3cEtQ3");

	testMenmonicToAddress("afloat afloat afloat abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"43FQ1sMFBAjFEazZhp9Z4593iJjhZdkaFFVDdGJmEp4YZUssYhxXM2E1Yrjqco3DRvLe84JWDXEMZSHzPxZXV4DXPuRaW68");

	testMenmonicToAddress("afoot afoot afoot abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"45dhHuMXLPNigk8X2xU521XBo2sSsXjLyMUe1nRhaE7VCVVD7VEaKtza5Eiz1QQ8C5M8yPe2qqZAd58kVsnCnfTvJZfpzZh");

	testMenmonicToAddress("afraid afraid afraid abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"45upjgnVH3bHybE2k4hWRh9hLZrVjsetL3zCmboKodzscUvc33CJ2avTE21M91hQkJdDbxCQbaHZPdnDDznZBxryJG5itaa");

	testMenmonicToAddress("after after after abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey after",
		"47CePGMwCnadnD8cJoiWLFdYUvT8kXwf17HfjKqEy6662h145iL35ei3AwbXuvucyX7WzzVndXD1RWFw3Uw7wD7YRwN9DnG");

	testMenmonicToAddress("against against against abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"49eSqQSLCMUAtT8BN9Gx8f5uaBNPwrzE7V4rvVXvhtbjNip5Ci2qkFRbqo8YhBBwC281EXqjFGzaRVGJez51r4EV6LC4Rko");

	testMenmonicToAddress("agenda agenda agenda abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"45XHgKT1SitKX2EvksLv32TBw2Mm1UedEcfEbD6D5ZGXMcLqQEZ7W5fiAWf4D8yUKeEWUxjCugRJhhCDVEf3vAGR7XEr1iU");

	testMenmonicToAddress("aggravate aggravate aggravate abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"41kc8R6Camw1KSyQCXduL21jp4LHPaCzqSAiATuAfp8FgzNiXXdSv9scWGacHqGSP114qPcYdAeivWbCntunMU5X9HZZhuq");

	testMenmonicToAddress("agile agile agile abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"49nXKL6pMEC6Xs2ySf7ysT3uyigUMLjW6CFjGYwAbdALQ9qGvbL8DrkB1fuLgE9ndNf539PVycrmRQvYNRbzHmhc4iFrJ7q");

	testMenmonicToAddress("aglow aglow aglow abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"47qEAEHtVbSRen1PyWm1gcHRxuxadR6LJRieZvXQ3LfkdkuaTRfwSqEXKNU4CdfTz3ByFZ3aNwshJCaV5wmZbcDGJdFVxsJ");

	testMenmonicToAddress("agnostic agnostic agnostic abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"47Fwhz45nbg6pXwoE8ktpD1CMSR33xrB4A7xqJEDPSupZcS7DfgR9oece4Rk3PLGUkA63Joz7SzcSEAzeTipbgLSQ5nYQo9");

	testMenmonicToAddress("agony agony agony abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"44bzyiyY8BWCUTE75nxpr4S9D5b9S9hQmbWFTFMpG7Y8eAzP5diKtEu2yViJts6vy7XWzCYYEbWZtQWuQfnQ6JjwHfg66mJ");

	testMenmonicToAddress("agreed agreed agreed abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"49S7HUBwVx75DtU2XFcWZQLMFBcDegeAQRSc1Be2YqugZZSYtTb52uW6QKZGAN7oKcY75S1DJmZS6XoPr6s2b9qBHhPDUzZ");

	testMenmonicToAddress("ahead ahead ahead abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey ahead",
		"44EQjM6NjEHNAM8ZJ8JunK55ioL2sfHfPDKLR21JD7PYhZ8xZF6M2JkUmUSf7GdhSX2Lws5Mnb4ax1vR12oVrdFEKVP7jkk");

	testMenmonicToAddress("aided aided aided abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey aided",
		"42KKNWo6vuq6oXJD4mrgAUHybGBpy2dGVJxPnLhqdWX7T9rMrKcLtS8VemVvGkhvKbiSzNDfWNd34D4fSj2YJxjJVPti3DV");

	testMenmonicToAddress("ailments ailments ailments abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4ABFx5Wn1fpBiGg2R2Mk2GSXhtUnWqVZMJeLqqgM3Tus7KTj8tBTre5jSFLUmE5GMbjiQjUh4PL7kKsqeSSzom7BLehguSi");

	testMenmonicToAddress("aimless aimless aimless abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4BE5A2L1EwbH4xsQmy75Wz74CPXGeTucbAP2q8JFM9hM3KVqFgjwxGfYxmD5SzgsUX52zP4i1qgwFNjCxLHaxrLKLrdvZXk");

	testMenmonicToAddress("airport airport airport abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey airport",
		"42nZkdj9ZxmbfznVXv4Dz28vkwsyXMGF5izc6E7dXEi47yK3ZHAMBza97nuZ7uJLpabPf98PLmaytTLYCx7nX26EF4oA7tt");

	testMenmonicToAddress("aisle aisle aisle abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"42JPUq4Goj2Pgrf9kRnRKBQGVT4dAkD2eVD3vu2mMaGzfRhKiLkvswbP1j711y5KpQ5T6ShRw38D69ABchg6FuuN3WpNjZH");

	testMenmonicToAddress("ajar ajar ajar abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"47sRUMRcQrMKAhv1asBs3AcLf91Sqsd3xU4Gy1D9KWrDJVdF9qqGz6cD2NU1DWtgaGQH7cFiC9jC3W6DP2B6W5T8PaSEUAo");

	testMenmonicToAddress("akin akin akin abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48MnSDXh2nVDJs581TKVqJj9fQYYZvDqaNsJgjESNBdX4We6oCdAFCq3pKY1QnW9W3Ur8odW9XHyve7t1mVuGPZWV9FfEzE");

	testMenmonicToAddress("alarms alarms alarms abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"491nvyEvZfDWFEfcAxbxupPnZswMgz6R7TaF5KnbZaXKQ64UUhHZtcgMC3CETZfoMD7QNAHW9B73C5RuzPBKUPLB9oi3fN8");

	testMenmonicToAddress("album album album abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"49yKdtVdXvvXDTeBprAaq3KwfuZJDYvSV6tNtBkL6uVuHcPFn4xHi8yRg73hs3x1FGLnZZN3cuderZ73gV6u3grL8eRJUSJ");

	testMenmonicToAddress("alchemy alchemy alchemy abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"45oMWXhUPVY347QPnX2dGGFYCT14CpWDyh9q4aV1aLHBC73ix6LXhgR3yXshGyoQvmTAWF42kEJcnXa126T9wXxs4w8QgEp");

	testMenmonicToAddress("alerts alerts alerts abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"44xAtYGdh9NWrDp4qD76tm4ZS4fLMpq1fheS4ws7Xt36UrsjoyCc9S5TjY8JJGom54SkxaZjyBfvA7NZCL1p4zWxDt8Mdio");

	testMenmonicToAddress("algebra algebra algebra abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"481xpnWFUHjQXUZ3SFDbvzj8V4e9yJtx15KMRoRiLPpiGtZR7EExtfEJtwrZ2xMHJqZfCePMM3M9AhFG7Uf2CwLXGxSJm6F");

	testMenmonicToAddress("alkaline alkaline alkaline abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"44DCdVVjxpHREpjAkAkCUsPWoGBqR4NzZZ6enXLdy2F6d5Yf5f9pa2mjKWamhiwWamfkUBZWfFRw73M4uPC3cTXFNxytNV7");

	testMenmonicToAddress("alley alley alley abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"47T6ki629SPfov4fUofY5gKZ77QDKW81fDpziaGYk7MsQKQebohVUeY4fG4A6YwD4dUkR6h7sKwUP41SXT3UrPEhMMNcWb2");

	testMenmonicToAddress("almost almost almost abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48Pg1HZuAW7CBS9d6Mrr3rGwy4eSvCE859pQAVS13YH5RTFEH3VMCctTWNV2VWcFoj9U8bLBggKnG4XCL3NZm8SnAcd1Uf7");

	testMenmonicToAddress("aloof aloof aloof abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48p9CJt78rrgixkGTipXo12xaXD27GhPP8VhwhPF8cGqETCUd9XEGEdVtHXZamBMXDYKcRP1onwSdQXERxYSZQyMVP2Ttgi");

	testMenmonicToAddress("alpine alpine alpine abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48TvroJLLgu1pTmW5XTtpuP773Npz5YNM1L4Lu2FfQZUbGt5b8w65YtQ1zPcshoQ11HpQiwJFpf1NBGmHVoGG6g449eYVv1");

	testMenmonicToAddress("already already already abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"49EfAt8tZ1dScegyfJCUR8LQ63Cc5xUrPV7gAurDNDQfHVHGiVaEe9K5GSQcEUpdGkSXXag9VNhg3Wq7z8afojwpMqPStxi");

	testMenmonicToAddress("also also also abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey also",
		"47HdG9DTS256zrMKpGoNd1GToMDXHXSWc1vVighfrdLa2eWhr6EzdDAgm67iKgZoeLHZsHtEogHX32qCqvMC8wEeTfYyYP4");

	testMenmonicToAddress("altitude altitude altitude abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"47a3C7CNsADaWkMksp3UoqWELhiVHWsXFDGkWppDSXyR4mRneXi41T4ZNebFKbungL79tczoUVSRbSWcqYjXeRAcSn6DkwW");

	testMenmonicToAddress("alumni alumni alumni abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"434iR9exqfKj432uScr6B2SeCRdeTzC8jL6TbtEazEwrYkH27Z4BYzti3ebZNJvDur6639CWdZ965HzRMKtWijiuG6aaeKi");

	testMenmonicToAddress("always always always abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"49nuAXLfGr7Bk9s2wBffoN4HrkdXrEwY5992jx6ZHL3h4ZD8dCNS9L9FKHLqPpUzu8f7UEgV1UJMwbUrAE67yTDyEKYwKiQ");

	testMenmonicToAddress("amaze amaze amaze abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4AnPiTvtehxfWY5paCtivRFigBZVMe33fPFyW2LH3dtU1iVhyYEmxwdGmucWKX1euYatVSB9ttR6MT87rMPARJ3r8yyqkkm");

	testMenmonicToAddress("ambush ambush ambush abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"43u5dEkbm77GvTBYrZZybkJnATEGvi5as6Tb6Hvnn5hYPkPqEPTuRVNcHa8ZWUtNNLFmEhwn29pPd7H3DHr2y1yL4ZUwV4Y");

	testMenmonicToAddress("amended amended amended abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"42K6GxAFoCcC9WV8f6HXZJMF8A2xhuedSRG9H7jEp71P7jk7t3G8WTSAUwU9WcVRPx2T6E28KfysFHn6HaLUXHmeNi5kxyb");

	testMenmonicToAddress("amidst amidst amidst abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"441QXJuXr3DBM2vVTveiJTKAr3nqbtAHH9dwLvHyU8UcTyb2Rg5xTpCJikRRTe9SjYcNzCPApdnW98Q3zoUaYKAmA6uRLXf");

	testMenmonicToAddress("ammo ammo ammo abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4ASZdbJoxDsLzkXBP8pjx2WZP7hqXwgxsWoY277AqtRx4gq3azDFL4LUVbmRewU4eAc3emSk1oWqpGojmMT9nqcg2aJXFCT");

	testMenmonicToAddress("amnesty amnesty amnesty abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48w8E2ixv9mZpXgU4c18BbZHhrKW2vKPm8dD5CoYEi6ra4qa1utdZBkQqc3KcXc21ndRMopPGdKERQx1HiWhgm66DCyvkHE");

	testMenmonicToAddress("among among among abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey among",
		"4AAr7VuZ1bNc4VrxsaAdHs8vv1i7ZiTvjiZybTKVVvqhFxSRGYaXT799ZvdHbEV1DBK1kfFoDnHNu9VQDRBdEr77J1oneaA");

	testMenmonicToAddress("amply amply amply abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"4ARWyN21F97XbhwRRS8eaZP62H2zJR8MBgeu99qqM14UWo5wobhsKs1MikHAFYBbvxRH23YbqqtPg8YcGC6rV3SXR3VCFEQ");

	testMenmonicToAddress("amused amused amused abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"42wUEuJzEzeSrU8G3nxZhkgwBUu5qzSLhS8AvF8eLsGNYd8Xi32ktjhctYsW8hHj9hf7ydunDAi9f5yCWJBiqnrDB34ZnFG");

	testMenmonicToAddress("anchor anchor anchor abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48EKTvhkmMnBMpSADnbQ92ZWpA83kub7fXBRkU2436r6ZYZ4dYYmiStHP9W5nQ8zmBiKE7g1C2B89KqtiLW3HB3Y73kUoeo");

	testMenmonicToAddress("android android android abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"429QEbuGmX9h9orywsm1DN6zxfHthtzefgxGvzANXauABQaHsy6oWtgR7a2wVPRZ3JHiAJNcLquwy6ikBSPR8bUuBs7z6kd");

	testMenmonicToAddress("anecdote anecdote anecdote abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey anecdote",
		"485hhm9CxoHE8XkRgT2iENhPmTDigC7HbfqPRWXiT1MZTAQaaKSbnDd7paJ4qeJ9xiZmAZ9tFPXdJVPPmFHAMxS4UvFXmuo");

	testMenmonicToAddress("angled angled angled abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"46dcpZHHTfL9mKhMm6hi6LNzQMAt9cZnE3DB1LYDNXoFUiXgZ5ZNf3CPo9UFTeGiDo9tGZeCVyGhM6hczCVfvqws8G4YEST");

	testMenmonicToAddress("ankle ankle ankle abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"44k6qMcFcSXHHMv3GTTiv3VSuDPMShYb3JDkHn76sgtrj6GMyF3WYVcAoFStZzakPzQ4Fvg8muUMgDUDgWXZkhS1Ers4Nw5");

	testMenmonicToAddress("annoyed annoyed annoyed abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48ePythWKMZNBue3fvxnYWTosDkgt4Tci8w9vquuyMVEi1mV7tEStQJP88sZs5fttj79vetB6bsjDYnVi1AiphoY2FrUxeY");

	testMenmonicToAddress("answers answers answers abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"477n9xJMsVKU6bdfCSo7NbND227ndj9pyiUfUnNW3cT2CATSNg8C1QHGZeH6uVwhjP97ahfKShDLY2TbX72CyzxXBTRCXhg");

	testMenmonicToAddress("antics antics antics abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48JV7vJDh1kYCFx7zYJJbsAYHM5oX3NXyFQwAgtN4wvM71i1bXwC7oi3EjubbHamem6bbVEKcFTGxEXdoW5khUNK1AiGzRF");

	testMenmonicToAddress("anvil anvil anvil abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48cjDQd4X16T3NhsmUAdxhZYwYRmfRWXQhpMQ8VPJHuR5qK8unnhbRq4pma66BGURYSwZddQVfwUwXPxVqvxccr7AZSpUha");

	testMenmonicToAddress("anxiety anxiety anxiety abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey anxiety",
		"48YUuqMmFNP3svxcgLSTfmAh6fyTBdny22W3kfAq1oMtAj1LkmWzR7QVWtLDZkpAWCKvymhPEry3VDiZhVWvfbZqJx2pRC3");

	testMenmonicToAddress("anybody anybody anybody abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"44nXm7xV6VnJuqCLEzaReP8iFNK79kWPHVbTuwGadcwcYw6QtuWB9rwREbEGh3afjR7DxXQgocovX1ZQuWMe3SxL4EqZfGG");

	testMenmonicToAddress("apart apart apart abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"46XjQ3LWvGGESKLYh5oM5KGwqP7BgGo95a7vwnHcNLkkCH3LwLzpWfWKejH8wYxfeDHuQMJbuufAWLBMZWow5cSQ4M17ooj");

	testMenmonicToAddress("apex apex apex abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey apex",
		"4AXWF1v3ibuDxhHh1ZC7fmVURDPBPxGYFj21wRztZuGVSAkAdYq4FYq6J3ZrcJwkq6eWXTgbEt1fp7DiVAjAd2VZUbVn24N");

	testMenmonicToAddress("aphid aphid aphid abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"47DsFswJjzzQrdAvh4eafd3VNAf9rKwdNYwrH1g5xM7pgPB8vqvMMp3BisK7Fwuw616n4MPMPm9LSM3czS3UvC8tUDFEApf");

	testMenmonicToAddress("aplomb aplomb aplomb abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"434JyLshfDxXRPG9fENXwjSpgPYrroPi9FuVxgCf57uV6i7eS3e2UX4H7fbC2hYL2ERd7824asPM493Za2tenyFtLQP74FX");

	testMenmonicToAddress("apology apology apology abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"47S8diEV9gn929faHWq575gUeyR3TXdp5RNMc7N4JXcjV14NaDS4zYtBytQbxTgfaeDmfBAyePknmdMrr7hX2hJwQqnb7Sy");

	testMenmonicToAddress("apply apply apply abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"48VqXPWtNLrE2tFWGkYHefMRSFnxooSf3AciF3UaECnohzpMwNgoSo22Af7rB5kRgRaFEzFxmam8EiFiLggbTocZAYWx38V");

	testMenmonicToAddress("apricot apricot apricot abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"478AoNHQNmAQU2mPk4sYVaHL1nvYik9JQYavo4smsSbQVmWRjZmmNN8AjhVqz2ghtvSZUnRnLmHKhUVxoPz7KQMYFJymgbZ");

	testMenmonicToAddress("aptitude aptitude aptitude abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey aptitude",
		"49xr5gHgh5Db8QiVMuoB2VW6KLGA9frUp8i93xpM38aF4xaZoD6HT1HKLKJ9zZu2fQK78qqPiK6yD1pQjWEaeyzA2qy256N");

	testMenmonicToAddress("aquarium aquarium aquarium abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey aquarium",
		"45eiyGhk7uE4GTpeH115cvS7EP9z9azV4A5i2mWQNGH8NRQUqVfz4Qs5G3pFf4yzXPh8YJpAPYVUCEa5vNx76hBm8pXnSMV");

	testMenmonicToAddress("arbitrary arbitrary arbitrary abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey arbitrary",
		"47LCmXZVEkgEr8HTNygeA79BsahmipoU3KjaTbTzBEPpDwmAfPfujFnjSaBf3p2kJzHhGtyD7NP3vQUP91M9DRShJ3f5D1Z");

	testMenmonicToAddress("archer archer archer abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"42ma7dmCtU3RbqMpomLENYJoiCteYvTEP4G3tKPoe45W5c5doiNJMtX3bsVqTa5Kvt1NRvLrLxqwdKqbayjz5S1PUouTkMP");

	testMenmonicToAddress("ardent ardent ardent abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"46HaqZCCqEzSERdyK5USUbakdofCFxD2y9ZvWxKFtXCA2yjNtmA9epAazHZQKLGCYJhvw9DkwMvRYRYX27CxVYtX5Dvm8y3");

	testMenmonicToAddress("arena arena arena abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey arena",
		"43jvcCDEyZk2iMNa6qNsmGjoZRpf5dXAFh46vQYYv2QaX9jVxdCZ1JkAn19AiRHUSPEVX4tRTWCfXfZsF1EtyLig6i3RiTs");

	testMenmonicToAddress("argue argue argue abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey abbey",
		"49EdZbhiNB9QFnG4YWBcwiaeiXd4YEs9xMMDq2441RUtbfwvsNQ7ZPegWnKK3SMjKaaMzMpojer3f14HpABGSr1EEhT2iiD");	

	testMenmonicToAddress("haggled saga nowhere olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept nowhere",
		"42SkcbytSLzAD1FSEkq1ELaKDWnqckQgfVXgwpTkH4ZZFsgLJPUqh5MDggLg3rqyZMSNnejwe19eSSrZjuX42X3MGwKpSKh");

	testMenmonicToAddress("hairy sailor nozzle olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept rigid",
		"46NWAeQRvZg5DWA4b7Z4KgbeZWgJMMD5mcrDcs1QuiyHfUkMYfWksudVWbAe1JZuSR2xroGSvh7U5ZZEn1E4qife3DGehQb");

	testMenmonicToAddress("hamburger sake nuance olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept ouch",
		"46sTsVbZu4eJTdkvmFKDJxJmT47b6raST3dQSqeJifTCYwUaPMEURAsY7fGpiNLkE8VDsx1VYcvvDLqtisfmogYnTiKhqEs");

	testMenmonicToAddress("happens salads nucleus olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept situated",
		"42wENRh7K6sLHbNvUo7C3V6g6rGoMGGLvYuBHtPHP5fH8YXEkPu9ubrCNnmQXhxUdWR4Gq3MELc1FRf7uA5gJDPhTC4tH55");

	testMenmonicToAddress("hashing sample nudged olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept nudged",
		"4B6JR97VDmd6qE51bNcuQQGDpK3PcSyj9MtriedVC6ZTdhuGzsLp9hpeGNPyDwj2kq5ht6qJQAg5nib2gCLzJLQ85MyLeJu");

	testMenmonicToAddress("hatchet sanity nugget olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept ouch",
		"49ig2wcXZ3M8eKdjP8xw9aQ5zpfEWywpfJfDgQFEpvjn7C4aFXC5ZzVTjhcVzAqP3UKauH85eC3y9MWvUWDotTdZCovDzLb");

	testMenmonicToAddress("haunted sapling nuisance olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept toaster",
		"48kFkiF1r8MHypGZWnB7eJ5fZtDupGMus8PbmUCVddfCDHN1q7pzX8PQpNMn1zdJRSPj39YNoC5xxdZDYWTUkqzy3wnRzu3");

	testMenmonicToAddress("having sarcasm null olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept setup",
		"48cG4QyHUCJczFJ6n7ukBvJEj7figUa6kXFPERowkCqjCrZWNser4yXQQjMbzzvwfiZLSLkoHAjgpTft8s4RhN6vBmjvKZm");

	testMenmonicToAddress("hawk sash number olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept toaster",
		"484LU4QRAujJFhkqUfizSkH8ez1LJa92q3dp7whEgVTZVKosKyvx8wqGngM9bMhb67G2CdGKkkypHbefqpqMhWJ2CWSxW5r");

	testMenmonicToAddress("haystack satin nuns olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept satin",
		"43yRsUCH3qZBjW8MBpdZ9jVSa9U3zPvwQ7RoMMFinFpMFvRpS9KQw41PHrpk45vuBqAS37ADBAfcLMcxHobwPnU3UwWAzog");

	testMenmonicToAddress("hazard saucepan nurse olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept dime",
		"458vDicBFjBYyMFYqiPQ33EYzpKvUzgR8i81JWs9VfZGgm5B1uppbfSCvy8wLFaGgTBGZk4xSmGYMhsnKtwbEQH1PsP5w5m");

	testMenmonicToAddress("hectare saved nutshell olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept hectare",
		"45fJfUFamfY28SgJqmFtPF7ECwbFctAU2fEVJJWdB1DrWZRorTghFwLJ7mQMMaWPadVX2VmCBN3zG82Ki2SeBnYLAjjKUY3");

	testMenmonicToAddress("hedgehog sawmill nylon olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept dodge",
		"46Wik48eDk4BK4aTRZams51ShKWxeR4NyKjiw76WUYZV3LciKB6vz3w3ZYHX3PwniUZABusAcBC7yZ1d8K8Y7pqHPKGSWAH");

	testMenmonicToAddress("heels saxophone oaks olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept sighting",
		"44LJQLUpFRdYQdA1ofAWhLBa5S2ftinEnPYoRqv9YFXZR1cUjdQjDqN4VqFm8ZHrvhZ1tGRfYGYZVBGUAZLH5FjgRGmfjEJ");

	testMenmonicToAddress("hefty sayings oars olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept swept",
		"4AWftMNGqc14JsXRLMRPziF6GFJa7bCMKJpXWuzyLSAuKdfNowiYJX7cJqe14FRuFCXbjzcoeGaY86EyhorLq1V4LHETh67");

	testMenmonicToAddress("height scamper oasis olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept situated",
		"497rRxcXcLVLyBWBL2j3RAJ8RzntTsCSYbAvx4y84Mt7MnRALG6ZaDjY5cJxJcF4pzMFdhTPJcZUGJWTAAGgTBUK5UFa3pa");

	testMenmonicToAddress("hemlock scenic oatmeal olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept popular",
		"45eYLEZLXa72VfiXS71EQo9zZtVG3sNg6R2Ckz5zvHwSbbVqByPwR8SZQE8nsuMVM3FUASZZpDzoM7rKdaxU3A6E4wAQgzx");

	testMenmonicToAddress("hence school obedient olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept rigid",
		"49ELLrTjNKt6qh6tcVAuTFDMaHtc4qSKoXNBvcundQ6bBEPsUwTw2QedRVSjZBaPLzR3rcv1wpfUd7fiW9YqWEfC9CjuSfX");

	testMenmonicToAddress("heron science object olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept sighting",
		"48X98qsaMD6HggCU4pDz8iG6JCL2J5DnY99zug9khkuj2VEoK2BNtJKhYMPPJUQdkharRnuSQsZN97YJy9vpnoQE9RT5YRL");

	testMenmonicToAddress("hesitate scoop obliged olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept scoop",
		"47m5GrhfDHY6L2gnyPM2iTgNFcmXuZvnUMDrqRQFNGu6R7ifuhVrYyZ9bVvs6avRrBfmhQqYT53kZdKYqJpNDu4ECcraV2f");

	testMenmonicToAddress("hexagon scrub obnoxious olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept incur",
		"45PMXD1Y92V3CRerLDA8vCQQHdBVZQtvS5cBmNPbR3oiL1ia37AotSxBtaYsoGkVcs6EXchzWnm42P9S2m2H1UMrBUmCQFx");

	testMenmonicToAddress("hickory scuba observant olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept scuba",
		"44G4N3suj4GTvo8pLmNTuE8piYdhhDRF9Hc7QoSwgkwz3HPkMvSkwjLYPct8T8aUdAenVu6EqZYzSWHAafhBnF14S3cCiZm");

	testMenmonicToAddress("hiding seasons obtains olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept setup",
		"48rh5C3EKMJKSPMRwB3kUSegYivYPmEavPgTodgQ1ei7AhCkgQWo8oDSPRqBsppv9NG2a8R11QiHX9QW4YJiwNw9BRxMBCN");

	testMenmonicToAddress("highway second obvious olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept sighting",
		"43DaMTc7HdL1fdH73QbGJB1L4qYdSFGyqXH8dQrwuLPbeX1WfnqLuAkCvScQ8kqL4vft9qpVDPWxdVwtGpK33yUYF9bKzi3");

	testMenmonicToAddress("hijack sedan occur olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept rigid",
		"46S91zTgnPCUm4fxj77Xqpe3Jex72syhs5ZvN2veqD3FSdwy5xXacgMVE48Dyr5LZEFaTEbgJiwgj5wx29Dh3FcR1kAZNi6");

	testMenmonicToAddress("hiker seeded ocean olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept ouch",
		"42q4AGjgTTHLYFWLLbKHjQGmkFf6t4nS2MAkfQc9Tzk5U76HTcu5jxKZNrEZ4LoHSbBsHA9P9X6MAL5GJCWB3EUC5qdEsAo");

	testMenmonicToAddress("hills segments october olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept sighting",
		"46RZV5aFh1WhzBDCkVnCq5ByZ1yL23ZQ57PqzsfPYUsCG5S2dNE2q3RGWtHCmkYdNbDdayo4dvwrNJgdF1DRQKSZ87Dv2Eh");

	testMenmonicToAddress("himself seismic odds olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept anvil",
		"43TPZqJbqYfP4x2RuRifsuGcM1i2DnaZFJPCtFtJauudP5hmiCC4i4u3hFJPBQm2qXipKKvDHMjbngtfDnqxqW9r9Lb7iZc");

	testMenmonicToAddress("hinder selfish odometer olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gourmet",
		"446KvKnASwh8PiAuCmx9BJHBZKMWzvtDUZsp6Lmxku3sMmfJM5wiufh335AqR6sFdkZfqWbk1cxtFPK9kjz5cFzsBeYktu5");

	testMenmonicToAddress("hippo semifinal offend olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept popular",
		"47uRpxgbLMBCkySH5FKVXYQWTyWMgZQ7aH1rVqgi8ypQY1xN1rXUyNhjU9syzRF6PigAtgD2oU1uH9gQjArSLQbk6PThmp1");

	testMenmonicToAddress("hire sensible often olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept hire",
		"4AYXDskvXTbcDKyiSo6SxRWHMWCQmvaoKcuGmUGKkKGbR95c9rxhLDcGcFkRkw4k7KiJP9nPUM1NagN7ppvmYEihJ4a6QWA");

	testMenmonicToAddress("history september oilfield olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept swept",
		"46RodLT72sRTuG6ma2nXMse8SSqq3zvSjKDn4pbakrd5DSVTfQTJFkiDP7gCjXsYwR7f66R7EQMbNGPJNugfZyo1AsWa8Fp");

	testMenmonicToAddress("hitched sequence ointment olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept incur",
		"43859iu9EeoLz83DXbtj37QUdgrawUrfrMv648YiXvRpFUKqGzen4nK9r4sv7eR7tjggvJRL84zaGRPCW73Y3QUVJC445QH");

	testMenmonicToAddress("hive serving okay olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept popular",
		"4AfQQejVLwF4wH8JvmZ2Jmae2y7Rwgp2ME35Z2Ewn6n79a3aFTQryhKEYWMAAGHpnqiLHhv5fJtqQZY5DkZ4BfBAHhZrJG1");

	testMenmonicToAddress("hoax session older olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept swept",
		"4ApvCT1DAKxBay4jG8eGULJErJSKNhedEK3yrTz3CUJ7WZkV4ijr9vEa4dedqajyJuRPiiMrQvriVYZ4x9qDaNJTT2gxJGF");

	testMenmonicToAddress("hobby setup olive olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gleeful",
		"49DJhTVkEMDNuztw8Q8gwfPzY9scy91To5qgx7HBXn4HSkYY7fenvmhB8aoYz562ze3KfzNhu4tRhYDGJ2czoUfQTpGQPJS");

	testMenmonicToAddress("hockey seventh olympics olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept olympics",
		"47FsHwa75hhEseH99BDEBpQPmpftbJ4eJXSMRR8Y6G1bECDt7L6ovpvCXVb15k9EkpSbzSSEv5WSCNHScJdgQP79TemSBQu");

	testMenmonicToAddress("hoisting sewage omega olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept incur",
		"48cgNAHFYbAFV3VCMzsnreNfDD8GQxp2HGdRsx3imfzV2hP9Qo46dbH3PGSRgWmw676jmzmbiFKteV3KVRL4w3xxVpoH5wS");

	testMenmonicToAddress("hold shackles omission olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept toaster",
		"473S9at4SgFVYpByvsAfNrZaqkv4HhsfTWozNVkB8DrfGkwFjBo8CLxZs2iTSyayXH3SiMFqmN1tGHeLb62uPyhCH6LLZtY");

	testMenmonicToAddress("honked shelter omnibus olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept popular",
		"47KRnfTwFxhdRThLyEv2K1fXovpGq5TP1NXqwqhXPygvZmnUNpxj5L23xa2m13NTj4h95pVC1pnpvX9FWMcn2wwd4QzsNDt");

	testMenmonicToAddress("hookup shipped onboard olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept toaster",
		"47XAC4XaBPb36FHxHGaDtmFJSSZj1r7qHjbmaKaG93PHNR163Ye4VhAed38DhuZePH9nye6wCuYp5ULpsv9F2oFESyV3537");

	testMenmonicToAddress("hope shocking oncoming olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept dodge",
		"46ef7yf8Ss61UqCVPjG1kwbhYDUy9V7crVRJCrPw3bKafSWpy1jfjKyDTFAmiBsSKzMCSGqspzMsE4fEP99vj9oLMCobWuN");

	testMenmonicToAddress("hornet shrugged oneself olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept incur",
		"42Us4Kg8iiT9UqgW2f1vpHN2987MPxZCbiYHrFu165yePnziEu3nDAFJTU5MZksGsrFGVJtF6vfMU2wFAUYALak8BRG1NMp");

	testMenmonicToAddress("hospital shuffled ongoing olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept dodge",
		"43bRaZ462wj7tPYC2GNGUsH4AALJZ9xCHLKaEDN3RajkZTUdLcxh9Nw5kenekrKcQKJJPeQENUiEpQqH1pCZkD5nMtuE1qr");

	testMenmonicToAddress("hotel shyness onion olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept popular",
		"48zfdfNSsa1jWrfMH7h8Gm6Y91yoNghUVGv4Cy7ydL9tTXPduTQdEdhVJTT5wtpW9ZNSfoNCHHP5c23sENkGFoDXD5qVyBY");

	testMenmonicToAddress("hounded siblings online olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept hounded",
		"435JcMCn4eqTQunhsyWVk8dqfQoeuRoKcD2PpgSWhXoFLXb3SJEDWLsEHLUNdoUxorcWNfHN5NJB2R2dRM7dwFxUReK4pUg");

	testMenmonicToAddress("hover sickness onslaught olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept swept",
		"45upWmuR53266VCdYoHrdobjShQZPvHnoCtoKmjh1Z7WW7jWntL5qn3UW92KxBaXxhb68P1ciSCdHaaZgRaE33iuKHynYwc");

	testMenmonicToAddress("howls sidekick onto olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept lair",
		"4B7fRwNV3hUKjGQ9SyDfCnHDfe6wwasSPYHBN1mvg5vtcowvxzyeyXQefNFcAzhgUS98w5moQHLRRFCMtCce3kegSxc7sXD");

	testMenmonicToAddress("hubcaps sieve onward olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept dodge",
		"44o8DPjchzjZprEC32T5ejEkzK59odAViLMeMzNBtfdCPsqE9YrVM3VfwU8FR79ptAJNTZCxtXeFGH6WKKm3g5YDCUTyYy1");

	testMenmonicToAddress("huddle sifting oozed olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept dodge",
		"42V1jsK5UKpJucaUDu1WeqDzXC16XQNNN7Gc1v2VpezeFKhkmRndnZs2eMmT6UrWqfEssYVkWwzUqQHnRzgDEhFDSicJbKE");

	testMenmonicToAddress("huge sighting opacity olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept ouch",
		"48jn8yYvMiifYKrGW8FYsQ5FqBZcJ1TPjFrYHrYwNcSCJhGf77UgrQ7cKiju8F56Vq1XP4ezMPx9XZaZiiHrzH7JATwjay4");

	testMenmonicToAddress("hull silk opened olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept setup",
		"437c4BPAThZTYuiEh5KRrs1dFxqQRqLBj3YGR4GjKLWuAwnDSLh9TdEeJddGahJ9gMg67jJp9GfZJ6qqd7mLrEtpJ2vKfTd");

	testMenmonicToAddress("humid simplest opposite olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept lair",
		"43ynWT6wT5dJnmhVYUpwwVPdcK39iy9fujdRLMiaqzEBUzWx2kZ3QfAWHA4t2LBdy6D2wPcgAU6gYaZHtzu4HpCB5qJq7Mc");

	testMenmonicToAddress("hunter sincerely optical olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept incur",
		"42W2KKRxAA2bWjUUeZvimAbU1ujgeX4FP4EfXxoJTdBFYsLwa5R2prTZhVD4Cj3wBjZkpnWoqFLM5PW6BTt7UmDY67t2Be9");

	testMenmonicToAddress("hurried sipped opus olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept sipped",
		"41efAPSd6XB5gpHUDMSgDJ415sZjxLcUjY6cSURtbT16Wj5Roji9Caj8LXNERbEfso74JTvbSXz7yeQum84pxRDKSYPWoJa");

	testMenmonicToAddress("husband siren orange olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept sighting",
		"41vbHHNdk5qK4qpUVZUuUoLQc38JzJDvvBZBVXXZThAmZqwFngSvVNP3bPUeFh6UwoLqc8W3xXdUxj15Woa2CeSRE6tAaVd");

	testMenmonicToAddress("huts situated orbit olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept dime",
		"41rqe6VanrtDqZkqyt8PHFWq7eTCTFHuNCkUZXSePPgTW9n2gsMVMEx49ksAekf731N4F5nnrhsbJSAGmdW5k6njCCfRauk");

	testMenmonicToAddress("hybrid sixteen orchid olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept olympics",
		"45L3KSBCTzRinnr1JFGuErX349P6HM3BafQ7AvMs5k8T1reWJJSUbRx3w7xqk6k9uZKczxx5HTKgXdJLBCpz642mSU9zGL8");

	testMenmonicToAddress("hydrogen sizes orders olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept hydrogen",
		"428qgY5kTTYKCnUBzNzmtRb5mjeLP5BqoSwa15hDsobPMCHYTuHPEtxBp64vXNm2VoKwHckxgThKKb2kKm1zmq2o17fjnR3");

	testMenmonicToAddress("hyper skater organs olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept rigid",
		"44GpcAsN5ByjjkohSH6APhcQd9KsNtcjCVKRB39q3GRvH9YrucHHAvRFuNJd84g9TNRpmCKi9aHXmRaGgfao9gw5DBojNNE");

	testMenmonicToAddress("iceberg skew origin olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gourmet",
		"45wqTxgQVbhYw3JJK1NaDzbnG8668r4g885M5k3TU2SRSRh1BxnbPNthEP6JuAnKaHZtFspng174ENX6fYfRAgPdMStveQL");

	testMenmonicToAddress("icing skirting ornament olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept rigid",
		"4B8NuRsFnRocUuDoar9iwjKTJ47WEDeFUWXt3nzSaDoca4qWj6BVc6EasoehxqvppMDHGXCNQ4kHNdRFm3pxrNM4EQLzvQR");

	testMenmonicToAddress("icon skulls orphans olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept situated",
		"49Kgm5nwuERSmjCnN3b1je8XBGfiebpGNMyvXnR24JAZ6M5rrUAFZQVjLXNDrRJszea3syFjErkjSTGeuQ9mZZSQ8CeGjQD");

	testMenmonicToAddress("identity skydive oscar olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gourmet",
		"46jEFgx36TM8UPyxpMnhhQANngLcQrqxpcbCn4oPLe3PAfcUUGFnbYYdmS5xdmzCiXiv16WGBaAhKE3b4d7dHUPk4rvmDMg");

	testMenmonicToAddress("idiom slackens ostrich olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept dodge",
		"43mGkcL8ccFTs79Y17bd4zX3MHmYxmggDSB1yBhG8MZg5cjERjVDcjjgCDi31CMnTyKEG5KZgGzvXZoak9tmXTAdN9MeMNt");

	testMenmonicToAddress("idled sleepless otherwise olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept swept",
		"47V4BjnQ8nCUcQtgK4ULe3NdKc8HcFjyYh4VJndXJSzLGNj9xzTub1VU4rMcQW423mAzg7HyqG1QnVWFu2aZeYXMUN8dDfs");

	testMenmonicToAddress("idols slid otter olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept olympics",
		"49paVFMVs6gHFTvup5bnpeb5Cv52aQhg6amWGRzMFPuUj94WgAqB4wtYXB5QemcE21SeJxtuQZkGt4C68SmKSaAw1NFx61C");

	testMenmonicToAddress("igloo slower ouch olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept etiquette",
		"47d2BYgDbxtTXG52TooWdMAXByadL7fpd9bgdoqvJH9dfZ8LxtGxrCvTASGftSrKNnMLmFY2pJUrNDCLWqwmBkbQA94bTmR");

	testMenmonicToAddress("ignore slug ought olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gleeful",
		"49w6NBRR7W2EL23yQ6FehaYpEC1pm4RCYeQYixoMeN2ZboZF8J4Z2tBUsTHxkTRSJTFp11QZW2XsTAhgRDmsvVYo3FY7urV");

	testMenmonicToAddress("iguana smash ounce olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept drunk",
		"44mb39VXg4WBCdhuRrWKdJdsimu2bq8GrVFyscrvhBM9ArHASX2Ge7e3Qu959ZuttV3RMRbSEFUV915y9jPpECdpNipZdaX");

	testMenmonicToAddress("illness smelting ourselves olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept anvil",
		"44PU4wu5jJzbsvy9QSxe3BVqt8d7tjPcZfA2GJS8qU9WAmEQpvqswKGRzWyq5pFMFA6UnYNKDenzFNPEVjm6bbUB3gBbpGE");

	testMenmonicToAddress("imagine smidgen oust olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept olympics",
		"42nhVDLRdYQJtE4MzvRZ2vMDJ9qjMgqdgb58BwkgJ5UudGifMdFZkHud9kNWjSPLsTRcs8gNSeYUiKKtmj7YWvk9BmGoQWm");

	testMenmonicToAddress("imbalance smog outbreak olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept elite",
		"4AU5tYxEiUhTf2kEfbucgodxURzDB1mNihzcS7VDcDK3iu1bDBRaq7pScStwpYufjnKHkhzdBJKBcYxKxhgLchzFL5hB3VU");

	testMenmonicToAddress("imitate smuggled oval olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept toaster",
		"4ApYVPYD3BX7VApU6GyaDR5k2XCHx6KZWSwDPxwTccH5NxDDGWDNXPN9PKVZKueHXQ6gqkwejv12w1D9dQg5f94eHWY6fgp");

	testMenmonicToAddress("impel snake oven olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept ouch",
		"4A68VYKs16mDCnQNf2e81ngQdCc9UtnWXUkvuesT8HCdErrxMyQxo6FM5uchYT7nuJ59RHkZ3fzpmcyG9tCC22rzUWuNi5T");

	testMenmonicToAddress("inactive sneeze owed olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept elite",
		"42XPnAGjayRhJp5XVr1YSsjoyaKEcP2ypMV81SY9XBUeTq6W7a2k9VF1fYyuhuh33iDRC94vZZD41iWj2etBKa1EHmvpeM2");

	testMenmonicToAddress("inbound sniff owls olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept dodge",
		"42JhBpTgLFhN9hsdWeLHvpAgB8tgFhBhSU9BywusBn53M3Z3gwhyNHXEXEHSaoTYLz35qE9JP6D4nEKbMYBGj14bV8LrdDW");

	testMenmonicToAddress("incur snout owner olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gleeful",
		"467TNHSNuVL3JvakLkhWY6ayHV3mM1NzzJ3qY4LiLfusaAEWNfBGQx6KzRWsdKG9KECtmdYTMZ3ywXjc9Qo5fPQWPwtXNjR");

	testMenmonicToAddress("industrial snug oxidant olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept snug",
		"4569kCA57EfYaQEHkFg87D8ZuwBB7CDiyFgyr5Dy55iaGRdWbrSE425iQPHvG95RTLVPEt33dvAP7X5M8sxgtTxZKXX7dTi");

	testMenmonicToAddress("inexact soapy oxygen olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept orchid",
		"48mnAzE78VJEoh7incWc4BRxNBUucKyZp28qiBdk4JNiUFvPu8nPcuXez4aanUJZeLG15xitbuguCScZZRCgaH4bFfgcFuN");

	testMenmonicToAddress("inflamed sober oyster olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept incur",
		"45p11bmxkekiDHvpPqhsEdRmAGWk6FN38SpSKRrT22bJeBXG9aSjjud45v3B2E24guc8rS6EWq5uf793CVnwTD8bF2pN6iS");

	testMenmonicToAddress("ingested soccer ozone olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept ingested",
		"436fQx2CE3S2VreTMLe1bkYPpdwqJoXQDghCniTPFxG92Xh6hd21XSVB91xrsDxewvBeMFcH9qNuZThyMnwadcrLDXYz3Aq");

	testMenmonicToAddress("initiate soda pact olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept drunk",
		"47ShMxaVzn3dsY7WP9LaLmHGVukEXJAXiYm3f6gfytesYNEaUR43rUKXXcBrdWYTQzcEjHqJY9mo8bfAFRutYudWRHRcVFQ");

	testMenmonicToAddress("injury software paddles olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept etiquette",
		"43fa54LswJkXw5NxwRNXad84jNRuQGHKRRUqce5b5JXS4KCot9P5EnSMZkWAiKpUzj1Aj48udChqTWMZhanUXtYV4obiZa1");

	testMenmonicToAddress("inkling soggy pager olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept pager",
		"43HvSjGya6TeYD1shpNwUjcXuWt793WAHUpRZxPwtg7B2xNVPc6rCh2d8VqjBcPYYZ5gBATSVbAZTUuzwgAJtTN35SDw3QE");

	testMenmonicToAddress("inline soil pairing olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gleeful",
		"47rmT9gtaZNCRFT2bDdtEpBFKHYcGSRo5NhMk2TM5D6dT5SQx8KSdsXDgpuC6mJ3W4PNaTEU2wgCa315uB6Woe939JRjMLv");

	testMenmonicToAddress("inmate solved palace olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept situated",
		"48NGGtbkhVZ6h5zwXJiE3c8BsuxCV6NRXT9NFRxT4iKXMz9BVsmPi8wKJB7podzue6LLDCAbPxgf487JuDj8hKaCVf7eaH3");

	testMenmonicToAddress("innocent somewhere pamphlet olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept situated",
		"4A7NjPy8oqy9rPbvLfdfvKTdJ3VwuzcbUUj8kFDhaCbqYgWDcX2uc5VbpJbftsPxhhPjA2yeU2xQtSGgTmEXyEQfBzcmand");

	testMenmonicToAddress("inorganic sonic pancakes olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept drunk",
		"44gHbsZPoiSgdCw88zfa3zdg9dnCeHLJpcH6pKz1SHnDCBJAb8S6Pm8NVR7M3p4Wbqdz5AoVPw4F4devGts6diM5Haus9z1");

	testMenmonicToAddress("input soothe paper olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept soothe",
		"47GJQ7chqGtDkxfW9sHgWPGyDfCjXQ2PbDL5DA7rp32qhuxPpbMp2zxW3kyXYm2rPR5ArqjpqREgJUDvWghDTp2TAX16GEn");

	testMenmonicToAddress("inquest soprano paradise olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gourmet",
		"4AWbUXWsd89PoPCiuAEGibSnVr9YBa3Fijadb6jGbFUdd58NNA42EF1M3e3j3hmasth622DELheRzXZD8pUA99UE5DrsbpT");

	testMenmonicToAddress("inroads sorry pastry olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept awesome",
		"44qyRxBYSUQEr988FNVLuxDfuq7v6jC3gNc22ioXYrok1oeMbLy7C11Z77x2yJHw3433VYC9LnbPf4A6WMW3JWRHNErs3aG");

	testMenmonicToAddress("insult southern patio olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept popular",
		"46CdH1nbrt3X9T2VVXyiwFbbvyxHfatQEDSVLveXVpcy4YpPmwuv4nG3KK9LtZLCvbYuAjsKcyRDc6EA79fu2PB29Tv6y1w");

	testMenmonicToAddress("intended sovereign pause olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept toaster",
		"46DQCBR86Thb1yvgJgqtzdBZtz7swDfmVNC53SejzQesYzgfXCTERNMVkj7yHkG2VxbWSyvQkrEQwbjDNeNktKmPEaytQsq");

	testMenmonicToAddress("inundate sowed pavements olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gourmet",
		"48eMx5qdwiX3tdLzCjQRWvXWR6W8Kh6eWicDU7eJ5eetUpEPTHta3Y39ZSBPcNg8vtjoZELgCiz3X1vMTuJhMem9VK7KdZW");

	testMenmonicToAddress("invoke soya pawnshop olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept gleeful",
		"46TGqJSa8exgkGQDL7uNoU12MDmrxzFDrUrhsbGrAaQwSoJRBJC7VYa7URdXrgwygpZ2NPX3gqg4fhVHeLeeVGLA2rP1kzN");

	testMenmonicToAddress("inwardly space payment olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept anvil",
		"489W7oY4KawdkqyFjEqWfCU92ceqHRydcbQt8PnRbHVzG2W2hkLgFzpXavPECjPMGRbbkJckyAinFdZrE3gC8mpJNzoqSKw");

	testMenmonicToAddress("ionic speedy peaches olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept rigid",
		"499UQayXUFDW6zkPYX54M4Z8ukv8ZSgBBXBVVZrQ8bqA8muuuf9FZMz9FnyiPRihaaeeH46A8vmPN6FPkeme2SyrRgDqiNB");

	testMenmonicToAddress("irate sphere pebbles olympics gleeful elite dodge lair dime awesome drunk setup incur toaster anvil etiquette gourmet popular rigid orchid sighting ouch situated swept lair",
		"44yrsvfTLvm4JxRP42KRaHfC8jUG1Kf7w2Tq1vgfVo1tUiZQ2anuVurabzkkUbCp3rWhBBDzXmDtM3Xr84jicaP94z8Hbad");
}


void generateNewTests()
{
	for(int i = 1 ; i < 100 ; i++)
	{
		const BigInt<256> base; //("B5EDDFC282B32945296CB8B318D980C8D0FDC36B9FECA84E7C4E355D72AB812");
		const BigInt<256> ii(i);
		const BigInt<256> privKey(base + ii);
	
		const BigInt<256> privKeyFinal = XmrEllipticCurve::getEdwards25519Max(privKey);
	
		const XmrPrivKey privkey(privKeyFinal);
		const XmrMnemonic mnemonicObj(privkey); //privKeyFinal);
	
		const XmrPrivKey myPrivKeySpend(privKeyFinal);
		const XmrPrivKey myViewKey = myPrivKeySpend.privateSpendToView();

		XmrEllipticCurve ec;

		const XmrPubKey pubKeySpend = myPrivKeySpend.getPubkey(ec);
		const XmrPubKey pubKeyView = myViewKey.getPubkey(ec);

		PubXmrAddress pubXmrAddress(pubKeySpend, pubKeyView);
				
		const std::string address = pubXmrAddress.toString();		
		
		std::cout << "\ttestMenmonicToAddress(\"" << mnemonicObj.toStr() << "\"," << std::endl;
		std::cout << "\t\t\"" << address << "\");" << std::endl;
		std::cout << std::endl;
	}
}

void moneroPointTests(const std::string& privNum, const std::string& expectedPoint)
{
	const XmrEllipticCurve ec;

	const BigInt<256> secret(privNum.c_str());

	std::cout << "Testing " << hexString(secret.getBinaryLong()) << std::flush;

	
	const BigInt<256> privKeyFinal = XmrEllipticCurve::getEdwards25519Max(secret);

	const XmrPrivKey privkey(privKeyFinal);

	const XmrECPoint point = privkey.point(ec);
	
	const std::string pointString = point.toString();
		
	if(pointString != expectedPoint)
	{
		std::cout << std::endl;		
		std::cout << "PrivNum : " << privNum << std::endl;
		std::cout << "Got     : " << pointString << std::endl;
		std::cout << "Expected: " << expectedPoint << std::endl;
		throw std::string("Test error");
	}

	std::cout << "......OK" << std::endl;
}


void moneroPubTest(const std::string& privNum, const std::string& expectedPub)
{
	const XmrEllipticCurve ec;

	const BigInt<256> secret(privNum.c_str());
	std::cout << "Testing " << hexString(secret.getBinaryLong()) << std::flush;
	
	const BigInt<256> privKeyFinal = XmrEllipticCurve::getEdwards25519Max(secret);

	const XmrPrivKey privkey(privKeyFinal);

	const XmrPubKey pub = privkey.getPubkey(ec);

	const std::string pubStr = pub.toString();
	
	//const std::string expectedPub("hdhfdjhsfjhfjkshdfkjshdfjkh");
		
	if(pubStr != expectedPub)
	{
		std::cout << std::endl;
		std::cout << "PrivNum : " << privNum << std::endl;
		std::cout << "Got     : " << pubStr << std::endl;
		std::cout << "Expected: " << expectedPub << std::endl;
		throw std::string("Test error");
	}

	std::cout << "......OK" << std::endl;
}



void moneroPrivTest(const std::string& privNum, const std::string& expectedPriv)
{
	const XmrEllipticCurve ec;

	const BigInt<256> secret(privNum.c_str());
	std::cout << "Testing " << hexString(secret.getBinaryLong()) << std::flush;
	
	const BigInt<256> privKeyFinal = XmrEllipticCurve::getEdwards25519Max(secret);

	const XmrPrivKey privkey(privKeyFinal);

	const std::string privStr = privkey.toBinaryXmrString();

	if(hexString(privStr) != expectedPriv)
	{
		std::cout << std::endl;
		std::cout << std::endl;		
		std::cout << "PrivNum : " << privNum << std::endl;
		std::cout << "Got     : " << hexString(privStr) << std::endl;
		std::cout << "Expected: " << expectedPriv << std::endl;
		throw std::string("Test error");
	}

	std::cout << "......OK" << std::endl;
}





void moneroECTest(const std::string& expectedPoint)
{
	const XmrEllipticCurve ec;
	
	const XmrECPoint newPoint1 = ec.addPoints(ec.identityPoint,  ec.genPoint);		
	const XmrECPoint newPoint2 = ec.addPoints(newPoint1,  ec.genPoint);		
	const XmrECPoint newPoint3 = ec.addPoints(newPoint2,  ec.genPoint);		
	
	std::cout << "New point" << newPoint3 << std::endl;
	
	if(newPoint3.toString() != expectedPoint)
	{
		std::cout << std::endl;
		std::cout << std::endl;

		std::cout << "Got     : " << newPoint3.toString() << std::endl;
		std::cout << "Expected: " << expectedPoint << std::endl;
		throw std::string("Test error");
	}
}




void moneroECTestIterative(const int numIterations, const std::string& expectedPoint)
{
	const XmrEllipticCurve ec;

	XmrECPoint newPoint(ec.identityPoint);	
	
	for(int i = 0 ; i < numIterations ; i++)
	{
		std::cout << "." << std::flush;
		if((i + 1) % 100 == 0)
		{
			std::cout << std::endl;
		}
		
		newPoint = ec.addPoints(newPoint,  ec.genPoint);
	}
	
	std::cout << "Final point" << newPoint << std::endl;
	
	if(newPoint.toString() != expectedPoint)
	{
		std::cout << std::endl;
		std::cout << std::endl;

		std::cout << "Got     : " << newPoint.toString() << std::endl;
		std::cout << "Expected: " << expectedPoint << std::endl;
		throw std::string("Test error");
	}
}



void moneroECTestMultiply(const int numIterations, const std::string& expectedPoint)
{
	const XmrEllipticCurve ec;

	//XmrECPoint newPoint(ec.identityPoint);	
	const BigInt<2048> iterations(numIterations);
			
	const XmrECPoint newPoint = ec.multiplyGenPoint(iterations);
	
	std::cout << "Final point" << newPoint << std::endl;
	
	if(newPoint.toString() != expectedPoint)
	{
		std::cout << std::endl;
		std::cout << std::endl;

		std::cout << "Got     : " << newPoint.toString() << std::endl;
		std::cout << "Expected: " << expectedPoint << std::endl;
		throw std::string("Test error");
	}
}





void moneroElipticCurveTests()
{
	moneroPointTests("823746237864237867237462384627846", 
		"(5D36465C1737364AF56ACB8B7FC5DCAE2CE9EDA929A7F5DC8BB5FA168BEE0026, 55DC5799E3572F8152DBC233E97ACD7C2F670C8A1E0A0E22478F2BAEB6B7A98F)");

	moneroPointTests("8237642786342786347263847623", 
		"(5C7FEE2758BCDC8BB161652E1197D30EF7E756340DE15D61EEFEB81C73CA2E19, 3E25AEA1E4A75EF7A172107BBA9E32089B62E2444CC66CDBBD3F8EF5BCD90EE)");

	moneroPointTests("9238742893748923748927348927348972983478237642786342786347263847", 
		"(194ECCFA5AD2DBE49C860D8BDB2FE128EB7C88AAE6F325BED5D9A1E3EEB6B16A, 1882E310A1204D19790408120C49472E847F490E59A537FB1AD456FC42C51445)");

	moneroPointTests("8374837483748374873483722983472893742389748748372487737377373771", 
		"(628663C49FF0387FABAF81DECE2330FA09FEE7EABAE0E53ED973DB7CB9123342, 3D8922858067F780E604AC9D37FD3A22EF7740958073738D50B921716AAFD910)");

	moneroPointTests("FFFFFCFCFCFFCFCFAFCAFCFAFCAFCFAFCAFACFACFAFCFCAFCFAFCFACFFCFFCFA", 
		"(296898A8F0389857192E3C01F7025F20BC5C09EFE178872CDF1EF1D89078BD9D, 1EC6AC560C42F4986875B20BF154333EB87098B4CB6954C6BDAAD8A75751A45A)");

	std::cout << "Pub tests" << std::endl;

	moneroPubTest("FFFFFCFCFC783384783747374387483748374873874738437847CFACFFCFFCFA", "39740B037FE1F88A3FE14816DB42F968964F05F6988DB806DAB0435BE9BEDAF8");

	std::cout << "Priv tests" << std::endl;


	moneroPrivTest("FFFFFCFCF7238472394732892374892789234789238472387477171711717111", "2E060A9F8B48634BA94102977DA312502689742389324739728423F7FCFCFF0F");

	moneroPrivTest("0000000000000000000000000000000723462364276472364723647623746631", "3166742376642347367264276423462307000000000000000000000000000000");

	moneroPrivTest("0000000000000000000000000000000777777777777777777777777777777777", "7777777777777777777777777777777707000000000000000000000000000000");


	moneroECTest("(67AE9C4A22928F491FF4AE743EDAC83A6343981981624886AC62485FD3F8E25C, 1267B1D177EE69ABA126A18E60269EF79F16EC176724030402C3684878F5B4D4)");

	moneroECTestIterative(1, "(216936D3CD6E53FEC0A4E231FDD6DC5C692CC7609525A7B2C9562D608F25D51A, 6666666666666666666666666666666666666666666666666666666666666658)");

	moneroECTestIterative(100, "(4B87A1147457B111116B878CFC2312DE451370AC38FE8690876EF6AC346FD47, 405EA0CDD414BDA960318B3108769A8928A25B756C372B254C69C78EA2FD81C5)");
	moneroECTestMultiply(100, "(4B87A1147457B111116B878CFC2312DE451370AC38FE8690876EF6AC346FD47, 405EA0CDD414BDA960318B3108769A8928A25B756C372B254C69C78EA2FD81C5)");

	moneroECTestIterative(500, "(5F5F78BBBBEF1B27052D345EDC5F82FF96FAE73F5DD7458BF5EB9598928B5326, 207C2509F5B2612D230019DE3ECD1434742FF71D78042A2694D2E49A2FDB6D7D)");
	moneroECTestMultiply(500, "(5F5F78BBBBEF1B27052D345EDC5F82FF96FAE73F5DD7458BF5EB9598928B5326, 207C2509F5B2612D230019DE3ECD1434742FF71D78042A2694D2E49A2FDB6D7D)");

	moneroECTestIterative(2000, "(6370847417FEF7A45596913B815A1BC844FD3585D0C283A64AF015855C248122, 3940AAFC43A361CBCE1465853B9AE085432C27413421606FF799C2AA1419BE5D)");
	moneroECTestMultiply(2000, "(6370847417FEF7A45596913B815A1BC844FD3585D0C283A64AF015855C248122, 3940AAFC43A361CBCE1465853B9AE085432C27413421606FF799C2AA1419BE5D)");


	moneroECTestMultiply(10000, "(6CD9B1B0F962CF2DEBB1B975683C7C3BA92CCAB561B30915CEA659579AE1478B, 7DC8FA47838698B5B6322A2052D5835BFD9497405CC53241337EF1115C4BE4BE)");
	moneroECTestMultiply(20000, "(7A01B6FD986771ED707A6CCA88FA10B144960A62FDEB33D9D6274984DCD8AF44, 16E045B129CE51CA0E315558F090FC7FBB1BFAE29F9A51C8B0BD2010DBE9EDE8)");
	moneroECTestMultiply(100000, "(65D1162A095D84BF680E3188022012EE4BB4B7804BF5D575766BCDB974672547, B72C2B5AECC2DDE6BC1F689EF601293098AB308AB4100379B948CA1FFCAD83B)");


}


void testGenAdress(const std::string& privNum, const std::string& expected)
{	
	BigInt<256> priv(privNum.c_str());
	const XmrPrivKey privKey(priv);
	std::cout << "Testing: " << privKey.toString() << "....." << std::flush;
	
	const PubXmrAddress address = privKey.getStandardAddress();
	std::cout << " " << address.toString() << " ";

	if(address.toString() != expected)
	{
		std::cout << std::endl;
		std::cout << std::endl;

		std::cout << "Got     : " << address.toString() << std::endl;
		std::cout << "Expected: " << expected << std::endl;
		throw std::string("Test error");
	}

	std::cout << "OK" << std::endl;
}





void testGenerationForAddresses()
{
	testGenAdress("981273891273123127837123781263781637861781263", "41nwZSbJneTAWgjUynQjUS1kxx8PyetJkddSFqoyPShyRxTPiptRWdQC4MMMr8e6c7X5ZgQXxKFg8iGszqxxj71jMHG6hrw");
	
	testGenAdress("6317378883738473874837438748374387437138278109747474747777747773", 
		"43Eyhk2y7CQd4z26TA873K5GT4p4mNTz3LhLqkoJVQbQWscwvWgL3MV1oepe8DFxQgCmNAAbFFDG9Hz9vk9TCVuYRJYqmCL");

	testGenAdress("63878787ABABBCABCBABCBABCBACBABCBABCBACBABCBBBBBBBCBCBBBBBCBBBBB", 
		"46aYJseeuBjNMTmMvMTJrqc3rEXQkLbdJ9KtmMUJ36gNixrJqLF9RNHKu1UD45QwcViXwNu3s6VJugBSmZu1wRKU3Rm42U5");

	testGenAdress("CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC", 
		"486g7ogswC9WygYYVZShfd2HPuG7uxWdG1M2rWAYFwggjLju6ohu9dtA2NqwkfW86XRAbYa7tSCE3FLUfZVRqT7rM9NyY6C");

	testGenAdress("1212121212121212121212121212121212121212121212121212121212121212", 
		"46hCrvFLEZ4CfGZXLjoaEbF8qiik6186UN1KPk98Rj3rCC5GW5omGkfQoQ6qnL33bvBzJoiqbtnFpaUu2KyLrefqBwBkqy7");

	testGenAdress("0000000000000000000000000000000000000000000000000000000000000001", 
		"44yQXfkWZNmJ8QgRfFWTzmJ8QgRfFWTzmJ8QgRfFWTzmJ9HskYAEKgjVy4kd3K4MaxERLtGa7FFrVNLF3jYWyjHCFA3tNHB");

	testGenAdress("0000000000000000000000000000000000000000000000000000000000000002", 
		"49GJVK3YQ7G3Q68xyoie3ASK9sBTXNue95yhG7PE7RLs6qtCWZpJ3NiUoUyBoqgURTKWYQumAc6EQYM4t4XQ7ZXyGidccNv");

	testGenAdress("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF", 
		"49voQEbjouUQSDikRWKUt1PGbS47TBde4hiGyftN46CvT7G378qfAdMRKMekVLopscBXNuY9N2tU99sn6Dcv2s5vFTnbQyA");

	testGenAdress("1111111111111111111111111111111111111111111111111111111111111111", 
		"46gXNFZinyUY2Zk5wJNro1L8GMSDCdCiF6rCZ3JKr3VJhffEiUm2AD7VBnsKejpwxwgcGr581mSkgDCorpbtNHHVMrVbtdi");

	testGenAdress("8888888888888888888888888888888888888888888888888888888888888888", 
		"4Abwd4tURxTUdxS3jWkST851bV9KV5zWw9H8vVEg4t7t2f4N2VLP5qYTyBuoef7mG8FddTucpMWU3ZWn6uv92pXvV69mjJ4");

	
}


void testGenSubAdress(const std::string& privNum, const int account, const int index, const std::string& expected)
{	
	BigInt<256> priv(privNum.c_str());
	const XmrPrivKey privKey(priv);
	std::cout << "Testing: " << privKey.toString() << " " << account << " " << index << "....." << std::flush;
		
	const XmrEllipticCurve ec;
	
	const PubXmrAddress address = privKey.getSubAddress(ec, account, index);

	std::cout << " " << address.toString() << " ";

	if(address.toString() != expected)
	{
		std::cout << std::endl;
		std::cout << std::endl;

		std::cout << "Got     : " << address.toString() << std::endl;
		std::cout << "Expected: " << expected << std::endl;
		throw std::string("Test error");
	}

	std::cout << "OK" << std::endl;
}


void testGenerationForSubAddresses()
{
	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 0, 0,
		"43Eyhk2y7CQd4z26TA873K5GT4p4mNTz3LhLqkoJVQbQWscwvWgL3MV1oepe8DFxQgCmNAAbFFDG9Hz9vk9TCVuYRJYqmCL");

	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 0, 1,
		"88z9khXi7V2CVFkUvuVdc2ddC1MumiJL2gfP9S6kWzVvF9BTUsik5W1B4oSe5nbsZmbGv64t7yMLZUpAqGXYn1T6Ay8oxNb");

	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 0, 5,
		"8B1siVm4fxPdnKzwk2yjxrA9WFbZySjxhKowmUqdtgJnE4TukxVPcKBHgV84qYX8LWK4oPBBXR2HKCz9a2CEm54vMeLVggT");

	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 0, 500,
		"87rfKGb2ur31dbkwghNNKceVCZCtmsDoj1MNVxPzsQX78iLhmLRc5nwFZ37E9j7tXeWz4etXV9Wms6ic94bJr9tG2uk2jUY");

	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 1, 0,
		"83kJpxM54XHXKhz414pHVo7aJarpGmfRaMrA85dBvHs4MhbMzv5CAbm1GD4iQxcwn4b1NwYYspZhsWbAfcBqfiz3GQJyveN");

	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 1, 10,
		"86DgECmjYZGgBVhVFgyb8MNXy1FrjgzxDYdNuczH9f37eEzLHY6E4smcPS5cmHSRvXhwRahaVQGjdiMd8T4nKLTV7ZkcMWQ");

	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 1, 100,
		"89Y9awTQz1BLamDjTexJ33XgZvvKBibNwiYLRv6YKX1YBGquFao35kP6N6gdv7QMZ9SWdynDALSKkGFGeCNiHvrYEE2GUAy");

	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 2, 10,
		"8C551gUMVHbLt7FtV8M4fq261p2uqFyYeZsDJwTniGKtBcEodZfPxtLPdYYmTv2wHJ6SX7BJQoSqi8Fv4KTwMLGnShjPpKS");

	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 1000, 200,
		"88r1oQQrUNG8Snbp1TNvcubbeygSXAbRHDrU7Cuj8ehSHVhEHVCdSEz8wwxvS47gWDVtvLcuKL38GYiFV2RvHbFh5nqkJpJ");

	testGenSubAdress("6317378883738473874837438748374387437138278109747474747777747773", 10000, 1234,
		"83ce7goLJsEQ91gtTcjPERXdtj5PeMzx11rroLjKRkVhDk7FASHuwpmCUkiLEmFZ88Q9usMoXh338XvbQJMRo2wdP1jVwfn");

	testGenSubAdress("6738754387438743874387384738743874384738478374837377838838388881", 10000, 1234,
		"88uSezCR7Sw8x4QoQH3S2bHtV4TWHjVoLaoWmkgtnYeqEqTryRvy2ReZztZY8EekNUWuTvKLHh2mwLAVzN3zA5fVB37eTkN");

	testGenSubAdress("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF", 100000, 100000,
		"8ArsCr8uZ3GbEmE3CDUpStiRDJJfJJWtDTtd7iDAyT7o8ScBFNaCF398RXJUtJrkehMM8XpCv6wsuW5EUSvZXV8ZQ2F6EoY");

}

void unitTestsMonero()
{
	//generateNewTests();
	//return;
		
	std::cout << "Unit tests for bitgen-xmr" << std::endl;

	moneroElipticCurveTests();
	
	testXmrMenmonicGeneration();

	testXmrPublicKeys();

	xmrUnittests();
	
	testMnemonicSeedsToAddresses();

	testGenerationForAddresses();
	
	testGenerationForSubAddresses();	
}








	
